@@ -1,9 +1,10 @@
-/* compile:
-clang++  -o http-server-1 http-server-1.cpp -I ~/projects/boost_1_73_0 -DWEBAPP_USES_RESOURCES -I. -fPIC -pthread -std=c++17 -Wall -g -DDEBUG -I ../../include/ -L ../../lib -lzeep-http -lzeep-xml -lzeep-json dummy_rsrc.o  -lstdc++fs
-*/
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
 
 // In this example we don't want to use rsrc based templates
-#undef WEBAPP_USES_RESOURCES
+#define WEBAPP_USES_RESOURCES 0
 
 //[ simple_http_server_2
 #include <zeep/http/server.hpp>
@@ -15,7 +16,7 @@ class hello_controller : public zeep::http::html_controller
   public:
     hello_controller()
     {
-        /*<< Mount the handler `handle_index` on =/=, =/index= and =/index.html= >>*/
+        /* Mount the handler `handle_index` on `/`, `/index` and `/index.html` */
         mount("{,index,index.html}", &hello_controller::handle_index);
     }
 
@@ -32,7 +33,7 @@ class hello_controller : public zeep::http::html_controller
 
 int main()
 {
-    /*<< Use the server constructor that takes the path to a docroot so it will construct a template processor >>*/
+    /* Use the server constructor that takes the path to a docroot so it will construct a template processor */
     zeep::http::server srv("docroot");
 
     srv.add_controller(new hello_controller());
