@@ -1,9 +1,16 @@
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
+
 // In this example we don't want to use rsrc based templates
-#undef WEBAPP_USES_RESOURCES
+#define WEBAPP_USES_RESOURCES 0
 
 #include <zeep/http/rest-controller.hpp>
 #include <zeep/http/html-controller.hpp>
 
+#include <algorithm>
+
 //[ cart_items
 struct Item
 {
@@ -117,7 +124,7 @@ class shop_html_controller : public zeep::http::html_controller
 //[ shop_main
 int main()
 {
-    /*<< Use the server constructor that takes the path to a docroot so it will construct a template processor >>*/
+    /* Use the server constructor that takes the path to a docroot so it will construct a template processor */
     zeep::http::server srv("docroot");
 
     srv.add_controller(new shop_html_controller());
@@ -130,4 +137,4 @@ int main()
 
     return 0;
 }
-//]
\ No newline at end of file
+//]
