@@ -1,9 +1,10 @@
-/* compile:
-clang++  -o security-sample security-sample.cpp -I ~/projects/boost_1_73_0 -DWEBAPP_USES_RESOURCES -I. -fPIC -pthread -std=c++17 -Wall -g -DDEBUG -I ../../include/ -L ../../lib -lzeep-http -lzeep-xml -lzeep-json dummy_rsrc.o  -lstdc++fs
-*/
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
 
 // In this example we don't want to use rsrc based templates
-#undef WEBAPP_USES_RESOURCES
+#define WEBAPP_USES_RESOURCES 0
 
 #include <zeep/http/server.hpp>
 #include <zeep/http/html-controller.hpp>
@@ -65,7 +66,7 @@ int main()
     //]
 
     //[ start_server
-    /*<< Use the server constructor that takes the path to a docroot so it will construct a template processor >>*/
+    /* Use the server constructor that takes the path to a docroot so it will construct a template processor */
     zeep::http::server srv(sc, "docroot");
 
     srv.add_controller(new hello_controller());
