@@ -1,4 +1,7 @@
-// compile: clang++ -o synopsis-json synopsis-json.cpp -I ../../include/  ../../lib/libzeep-json.a ../../lib/libzeep-generic.a -std=c++17 -lstdc++fs -I ~/projects/boost_1_73_0/
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
 
 #include <iostream>
 #include <cassert>
@@ -12,7 +15,7 @@ int main()
     zeep::http::scope scope;
 
     //[ fill_scope
-    /*<< Fill a scope with an array of objects, each object having one element >>*/
+    /* Fill a scope with an array of objects, each object having one element */
     zeep::json::element ints{
         {
             { "value", 1 }
@@ -28,7 +31,7 @@ int main()
     auto s = zeep::http::evaluate_el(scope, "|1: ${ints[0].value}, 2: ${ints[1].value}|");
     //]
 
-    std::cout << s << std::endl;
+    std::cout << s << '\n';
 
     assert(s == "1: 1, 2: 2");
 
