@@ -1,3 +1,9 @@
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
+
+#include <iostream>
 #include <zeep/xml/document.hpp>
 
 //[ synopsis_xml_main
@@ -5,7 +11,7 @@ int main()
 {
     using namespace zeep::xml::literals; 
 
-    /*<< Construct an XML document in memory using a string literal >>*/
+    /* Construct an XML document in memory using a string literal */
     auto doc = 
         R"(<persons>
             <person id="1">
@@ -18,19 +24,19 @@ int main()
             </person>
         </persons>)"_xml;
 
-    /*<< Iterate over an XPath result set >>*/
+    /* Iterate over an XPath result set */
     for (auto& person: doc.find("//person")) 
     {
         std::string firstname, lastname;
 
-        /*<< Iterate over the __element__ nodes inside the person __element__ >>*/
+        /* Iterate over the __element__ nodes inside the person __element__ */
         for (auto& name: *person)
         {
             if (name.name() == "firstname")	firstname = name.str();
             if (name.name() == "lastname")	lastname = name.str();
         }
 
-        std::cout << person->get_attribute("id") << ": " << lastname << ", " << firstname << std::endl;
+        std::cout << person->get_attribute("id") << ": " << lastname << ", " << firstname << '\n';
     }
 
     return 0;
