@@ -1,9 +1,11 @@
-// compile: clang++ -o validating-xml-sample validating-xml-sample.cpp -I ../../include/  ../../lib/libzeep-xml.a ../../lib/libzeep-generic.a -std=c++17 -lstdc++fs -I ~/projects/boost_1_73_0/ && ./validating-xml-sample
-
-//[ xml_validation_sample
+//           Copyright Maarten L. Hekkelman, 2022-2023
+//  Distributed under the Boost Software License, Version 1.0.
+//     (See accompanying file LICENSE_1_0.txt or copy at
+//           http://www.boost.org/LICENSE_1_0.txt)
 
 #include <filesystem>
 #include <fstream>
+#include <iostream>
 
 #include <zeep/xml/document.hpp>
 
@@ -11,7 +13,8 @@ namespace fs = std::filesystem;
 
 int main()
 {
-    /*<< Define an entity loader function >>*/
+//[ xml_validation_sample
+    /* Define an entity loader function */
     auto loader = []
         (const std::string& base, const std::string& pubid, const std::string& sysid) -> std::istream*
     {
@@ -21,21 +24,20 @@ int main()
         throw std::invalid_argument("Invalid arguments passed in loader");
     };
 
-    /*<< Create document and set the entity loader >>*/
+    /* Create document and set the entity loader */
     zeep::xml::document doc;
     doc.set_entity_loader(loader);
 
-    /*<< Read a file >>*/
+    /* Read a file */
     std::ifstream is("sample.xml");
     is >> doc;
 
     using namespace zeep::xml::literals;
 
-    /*<< Compare the doc with an in-memory constructed document, note that spaces are ignored >>*/
+    /* Compare the doc with an in-memory constructed document, note that spaces are ignored */
     if (doc == R"(<foo><bar>Hello, world!</bar></foo>)"_xml)
-        std::cout << "ok" << std::endl;
+        std::cout << "ok\n";
+//]
 
     return 0;
 }
-
-//]
\ No newline at end of file
