@@ -1,4 +1,4 @@
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //  Distributed under the Boost Software License, Version 1.0.
 //     (See accompanying file LICENSE_1_0.txt or copy at
 //           http://www.boost.org/LICENSE_1_0.txt)
@@ -36,19 +36,19 @@ class controller
 	virtual ~controller();
 
 	/// \brief Calls handle_request but stores a pointer to the request first
-	virtual bool dispatch_request(boost::asio::ip::tcp::socket &socket, request &req, reply &rep);
+	virtual bool dispatch_request(asio_ns::ip::tcp::socket &socket, request &req, reply &rep);
 
 	/// \brief The pure virtual method that actually handles the request
 	virtual bool handle_request(request &req, reply &rep) = 0;
 
 	/// \brief returns the defined prefix path
-	std::string get_prefix() const { return m_prefix_path; }
+	uri get_prefix() const { return m_prefix_path; }
 
 	/// \brief return whether this uri request path matches our prefix
-	bool path_matches_prefix(const std::string &path) const;
+	bool path_matches_prefix(const uri &path) const;
 
 	/// \brief return the path with the prefix path stripped off
-	std::string get_prefixless_path(const request &req) const;
+	uri get_prefixless_path(const request &req) const;
 
 	/// \brief bind this controller to \a server
 	virtual void set_server(basic_server *server)
@@ -57,8 +57,14 @@ class controller
 	}
 
 	/// \brief return the server object we're bound to
-	const basic_server &get_server() const { return *m_server; }
-	basic_server &get_server() { return *m_server; }
+	const basic_server *get_server() const { return m_server; }
+	basic_server *get_server() { return m_server; }
+
+	/// \brief return the context name, if specified. Empty string otherwise
+	std::string get_context_name() const
+	{
+		return m_server ? m_server->get_context_name() : "";
+	}
 
 	/// \brief get the credentials for the current request
 	json::element get_credentials() const;
@@ -72,11 +78,14 @@ class controller
 	/// \brief return a specific header line from the original request
 	std::string get_header(const char *name) const;
 
+	/// \brief Fill in the OPTIONS in reply \a rep for a request \a req
+	virtual void get_options(const request &req, reply &rep);
+
   protected:
 	controller(const controller &) = delete;
 	controller &operator=(const controller &) = delete;
 
-	std::string m_prefix_path;
+	uri m_prefix_path;
 	basic_server *m_server = nullptr;
 	static thread_local request *s_request;
 };
