@@ -1,4 +1,4 @@
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //  Distributed under the Boost Software License, Version 1.0.
 //     (See accompanying file LICENSE_1_0.txt or copy at
 //           http://www.boost.org/LICENSE_1_0.txt)
@@ -12,6 +12,7 @@
 
 #include <zeep/config.hpp>
 
+#include <cstdint>
 #include <string>
 #include <functional>
 
@@ -38,9 +39,9 @@ class daemon
 	/// \brief constructor with separately specified files
 	///
 	/// \param factory			The function object that creates server instances
-	/// \param pid_file			The file that will contain the process ID, usually in /var/run/<process_name>
-	/// \param stdout_log_file	The file that will contain the stdout log, usually in /var/log/<process_name>/access.log
-	/// \param stderr_log_file	The file that will contain the stderr log, usually in /var/log/<process_name>/error.log
+	/// \param pid_file			The file that will contain the process ID, usually in /var/run/{process_name}
+	/// \param stdout_log_file	The file that will contain the stdout log, usually in /var/log/{process_name}/access.log
+	/// \param stderr_log_file	The file that will contain the stderr log, usually in /var/log/{process_name}/error.log
 	daemon(server_factory_type&& factory, const std::string& pid_file,
 		const std::string& stdout_log_file, const std::string& stderr_log_file);
 
@@ -61,7 +62,21 @@ class daemon
 		m_restart_time_window = within_nr_of_seconds;
 	}
 
-	/// \brief Start the daemon, forking off in the background
+#if HTTP_HAS_UNIX_DAEMON
+
+	/// \brief Start the daemon, forking off in the background with multiple preforked servers
+	///
+	/// \param address				The address to bind to
+	/// \param port					The port number to bind to
+	/// \param nr_of_procs			The number of worker processes to fork
+	/// \param nr_of_threads		The number of threads to pass to the server class
+	/// \param run_as_user			The user to run the forked process. Daemons are usually
+	///								started as root and should drop their privileges as soon
+	///								as possible.
+	int start(const std::string& address, uint16_t port, size_t nr_of_procs,
+		size_t nr_of_threads, const std::string& run_as_user);
+
+	/// \brief Start the daemon, forking off in the background with single process
 	///
 	/// \param address				The address to bind to
 	/// \param port					The port number to bind to
@@ -69,8 +84,8 @@ class daemon
 	/// \param run_as_user			The user to run the forked process. Daemons are usually
 	///								started as root and should drop their privileges as soon
 	///								as possible.
-	int start(const std::string& address, uint16_t port, size_t nr_of_procs,
-		size_t nr_of_threads, const std::string& run_as_user);
+	int start(const std::string& address, uint16_t port, size_t nr_of_threads,
+		const std::string& run_as_user);
 
 	/// \brief Stop a running daemon process. Returns 0 in case of successfully stopping a process.
 	int stop();
@@ -81,6 +96,8 @@ class daemon
 	/// \brief Force the running daemon to restart
 	int reload();
 
+#endif
+
 	/// \brief Run the server without forking to the background
 	///
 	/// For debugging purposes it is sometimes useful to start a server
@@ -90,12 +107,16 @@ class daemon
 
   private:
 
-	void daemonize();
+#if HTTP_HAS_UNIX_DAEMON
+
+	int daemonize();
 	void open_log_file();
+
 	bool run_main_loop(const std::string& address, uint16_t port, size_t nr_of_procs,
 		size_t nr_of_threads, const std::string& run_as_user);
 
 	bool pid_is_for_executable();
+#endif
 
   private:
 	server_factory_type m_factory;
