@@ -1,5 +1,5 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -80,7 +80,7 @@ class preforked_server
 
     std::function<basic_server*(void)>	m_constructor;
     std::mutex						m_lock;
-	boost::asio::io_context			m_io_context;
+	asio_ns::io_context			m_io_context;
 };
 
 }
