@@ -1,4 +1,4 @@
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //   Distributed under the Boost Software License, Version 1.0.
 //      (See accompanying file LICENSE_1_0.txt or copy at
 //            http://www.boost.org/LICENSE_1_0.txt)
@@ -10,13 +10,15 @@
 
 #include <zeep/config.hpp>
 
-#include <set>
-
-#include <zeep/exception.hpp>
 #include <zeep/crypto.hpp>
+#include <zeep/exception.hpp>
 #include <zeep/http/server.hpp>
 #include <zeep/json/element.hpp>
 
+#include <set>
+
+#include <cassert>
+
 // --------------------------------------------------------------------
 //
 
@@ -30,31 +32,41 @@ namespace zeep::http
 struct unauthorized_exception : public zeep::exception
 {
 	/// \brief constructor
-	unauthorized_exception() : exception("unauthorized") { }
+	unauthorized_exception()
+		: exception("unauthorized")
+	{
+	}
 };
 
 // --------------------------------------------------------------------
 
+
+/// \brief Base class for password encoders
 class password_encoder
 {
   public:
 	virtual ~password_encoder() {}
 
-	virtual std::string encode(const std::string& password) const = 0;
-	virtual bool matches(const std::string& raw_password, const std::string& stored_password) const = 0;
+	virtual std::string encode(const std::string &password) const = 0;
+	virtual bool matches(const std::string &raw_password, const std::string &stored_password) const = 0;
 };
 
 // --------------------------------------------------------------------
 
+/// \brief Implementation of @ref zeep::http::password_encoder for the PBKDF2-SHA256 algorithm
+/// https://en.wikipedia.org/wiki/PBKDF2
 class pbkdf2_sha256_password_encoder : public password_encoder
 {
   public:
-	static inline constexpr const char* name() { return "pbkdf2_sha256"; };
+	static inline constexpr const char *name() { return "pbkdf2_sha256"; };
 
 	pbkdf2_sha256_password_encoder(int iterations = 30000, int key_length = 32)
-		: m_iterations(iterations), m_key_length(key_length) {}
+		: m_iterations(iterations)
+		, m_key_length(key_length)
+	{
+	}
 
-	virtual std::string encode(const std::string& password) const
+	virtual std::string encode(const std::string &password) const
 	{
 		using namespace std::literals;
 
@@ -63,7 +75,7 @@ class pbkdf2_sha256_password_encoder : public password_encoder
 		return "pbkdf2_sha256$" + std::to_string(m_iterations) + '$' + salt + '$' + pw;
 	}
 
-	virtual bool matches(const std::string& raw_password, const std::string& stored_password) const
+	virtual bool matches(const std::string &raw_password, const std::string &stored_password) const
 	{
 		using namespace std::literals;
 
@@ -100,8 +112,12 @@ class pbkdf2_sha256_password_encoder : public password_encoder
 struct user_details
 {
 	user_details() {}
-	user_details(const std::string& username, const std::string& password, const std::set<std::string>& roles)
-		: username(username), password(password), roles(roles) {}
+	user_details(const std::string &username, const std::string &password, const std::set<std::string> &roles)
+		: username(username)
+		, password(password)
+		, roles(roles)
+	{
+	}
 
 	std::string username;
 	std::string password;
@@ -110,18 +126,27 @@ struct user_details
 
 // --------------------------------------------------------------------
 
-/// \brief exception thrown by user_service when trying to load user_details for an unknown user
-class user_unknown_exception : public zeep::exception
+class authentication_exception : public zeep::exception
 {
   public:
-	user_unknown_exception() : zeep::exception("user unknown") {};
+	authentication_exception(const char *msg)
+		: zeep::exception(msg) {}
+};
+
+/// \brief exception thrown by user_service when trying to load user_details for an unknown user
+class user_unknown_exception : public authentication_exception
+{
+  public:
+	user_unknown_exception()
+		: authentication_exception("user unknown"){};
 };
 
 /// \brief exception thrown by security_context when a username/password combo is not valid
-class invalid_password_exception : public zeep::exception
+class invalid_password_exception : public authentication_exception
 {
   public:
-	invalid_password_exception() : zeep::exception("invalid password") {};
+	invalid_password_exception()
+		: authentication_exception("invalid password"){};
 };
 
 // --------------------------------------------------------------------
@@ -137,7 +162,13 @@ class user_service
 	virtual ~user_service() {}
 
 	/// \brief return the user_details for a user named \a username
-	virtual user_details load_user(const std::string& username) const = 0;
+	virtual user_details load_user(const std::string &username) const = 0;
+
+	/// \brief return true if the credentials in \a credentials are still sufficient to access this web application
+	virtual bool user_is_valid(const json::element &credentials) const;
+
+	/// \brief return true if a user named \a username is allowed to access this web application
+	virtual bool user_is_valid(const std::string &username) const;
 };
 
 // --------------------------------------------------------------------
@@ -150,23 +181,24 @@ class user_service
 class simple_user_service : public user_service
 {
   public:
-	simple_user_service(std::initializer_list<std::tuple<std::string,std::string,std::set<std::string>>> users)
+	simple_user_service(std::initializer_list<std::tuple<std::string, std::string, std::set<std::string>>> users)
 	{
-		for (auto const& [username, password, roles]: users)
+		for (auto const &[username, password, roles] : users)
 			add_user(username, password, roles);
 	}
 
 	/// \brief return the user_details for a user named \a username
-	virtual user_details load_user(const std::string& username) const
+	virtual user_details load_user(const std::string &username) const
 	{
 		user_details result = {};
-		auto ui = std::find_if(m_users.begin(), m_users.end(), [username](const user_details& u) { return u.username == username; });
+		auto ui = std::find_if(m_users.begin(), m_users.end(), [username](const user_details &u)
+			{ return u.username == username; });
 		if (ui != m_users.end())
 			result = *ui;
 		return result;
 	}
 
-	void add_user(const std::string& username, const std::string& password, const std::set<std::string>& roles)
+	void add_user(const std::string &username, const std::string &password, const std::set<std::string> &roles)
 	{
 		m_users.emplace_back(username, password, roles);
 	}
@@ -192,17 +224,13 @@ class security_context
 	///
 	/// Create a security context for server \a s with validator \a validator and
 	/// a flag \a defaultAccessAllowed indicating if non-matched uri's should be allowed
-	security_context(const std::string& secret, user_service& users, bool defaultAccessAllowed = false)
-		: m_secret(secret), m_users(users), m_default_allow(defaultAccessAllowed)
-	{
-		register_password_encoder<pbkdf2_sha256_password_encoder>();
-	}
+	security_context(const std::string &secret, user_service &users, bool defaultAccessAllowed = false);
 
 	/// \brief register a custom password encoder
 	///
 	/// The password encoder should derive from the abstract password encoder class above
 	/// and also implement the name() method.
-	template<typename PWEncoder>
+	template <typename PWEncoder>
 	void register_password_encoder()
 	{
 		m_known_password_encoders.emplace_back(PWEncoder::name(), new PWEncoder());
@@ -213,8 +241,8 @@ class security_context
 	/// A new rule will be added to the list, allowing access to \a glob_pattern
 	/// to users having role \a role
 	///
- 	/// \a glob_pattern should start with a slash
-	void add_rule(const std::string& glob_pattern, const std::string& role)
+	/// \a glob_pattern should start with a slash
+	void add_rule(const std::string &glob_pattern, const std::string &role)
 	{
 		add_rule(glob_pattern, { role });
 	}
@@ -230,7 +258,7 @@ class security_context
 	void add_rule(std::string glob_pattern, std::initializer_list<std::string> roles)
 	{
 		assert(glob_pattern.front() == '/');
-		m_rules.emplace_back(rule{glob_pattern, roles});
+		m_rules.emplace_back(rule{ glob_pattern, roles });
 	}
 
 	/// \brief Validate the request \a req against the stored rules
@@ -239,16 +267,7 @@ class security_context
 	/// and will throw an exception if access is not allowed.
 	/// The request \a req will be updated with the credentials for further use.
 	/// If the validate CSRF is set, the CSRF token will also be validated.
-	void validate_request(request& req) const;
-
-	/// \brief Add e.g. headers to reply for an authorized request
-	///
-	/// When validation succeeds, a HTTP reply is send to the user and this routine will be
-	/// called to augment the reply with additional information.
-	///
-	/// \param rep			Then zeep::http::reply object that will be send to the user
-	/// \param username		The name for the authorized user, credentials will be fetched from the user_service
-	void add_authorization_headers(reply &rep, const std::string& username);
+	void validate_request(request &req) const;
 
 	/// \brief Add e.g. headers to reply for an authorized request
 	///
@@ -259,6 +278,17 @@ class security_context
 	/// \param user			The authorized user details
 	void add_authorization_headers(reply &rep, const user_details user);
 
+	/// \brief Add e.g. headers to reply for an authorized request, with an expiration parameter
+	///
+	/// When validation succeeds, a HTTP reply is send to the user and this routine will be
+	/// called to augment the reply with additional information.
+	///
+	/// \param rep			Then zeep::http::reply object that will be send to the user
+	/// \param user			The authorized user details
+	/// \param exp			The maximum lifetime for the access token
+	void add_authorization_headers(reply &rep, const user_details user,
+		std::chrono::system_clock::duration exp);
+
 	/// \brief verify the username/password combination and set a cookie in the reply in case of success
 	///
 	/// When validation succeeds, add_authorization_headers is called, otherwise an exception is thrown.
@@ -266,39 +296,49 @@ class security_context
 	/// \param username		The name for the user
 	/// \param password		The password for the user
 	/// \param rep			Then zeep::http::reply object that will be send back to the browser
-	void verify_username_password(const std::string& username, const std::string& password, reply &rep);
+	void verify_username_password(const std::string &username, const std::string &password, reply &rep);
+
+	/// \brief verify the username/password combination and return true if valid
+	///
+	/// \param username		The name for the user
+	/// \param password		The password for the user
+	/// \result             True in case of valid combination
+	bool verify_username_password(const std::string &username, const std::string &password);
 
 	/// \brief return reference to the user_service object
-	user_service& get_user_service() const			{ return m_users; }
+	user_service &get_user_service() const { return m_users; }
 
 	/// \brief Get or create a CSRF token for the current request
 	///
 	/// Return a CSRF token. If this was not present in the request, a new will be generated
 	/// \param req		The HTTP request
 	/// \return			A std::pair containing the CSRF token and a flag indicating the token is new
-	std::pair<std::string,bool> get_csrf_token(request& req);
+	std::pair<std::string, bool> get_csrf_token(request &req);
 
 	/// \brief To automatically validate CSRF tokens, set this flag
-	void set_validate_csrf(bool validate)			{ m_validate_csrf = validate; }
-	bool get_validate_csrf() const					{ return m_validate_csrf; }
+	void set_validate_csrf(bool validate) { m_validate_csrf = validate; }
+	bool get_validate_csrf() const { return m_validate_csrf; }
+
+	std::chrono::system_clock::duration get_jwt_exp() const { return m_default_jwt_exp; }
+	void set_jwt_exp(std::chrono::system_clock::duration exp) { m_default_jwt_exp = exp; }
 
   private:
-	security_context(const security_context&) = delete;
-	security_context& operator=(const security_context&) = delete;
+	security_context(const security_context &) = delete;
+	security_context &operator=(const security_context &) = delete;
 
 	struct rule
 	{
-		std::string				m_pattern;
-		std::set<std::string>	m_roles;
+		std::string m_pattern;
+		std::set<std::string> m_roles;
 	};
 
 	std::string m_secret;
-	user_service& m_users;
+	user_service &m_users;
 	bool m_default_allow;
 	bool m_validate_csrf = false;
 	std::vector<rule> m_rules;
-	std::vector<std::tuple<std::string,std::unique_ptr<password_encoder>>> m_known_password_encoders;
+	std::vector<std::tuple<std::string, std::unique_ptr<password_encoder>>> m_known_password_encoders;
+	std::chrono::system_clock::duration m_default_jwt_exp;
 };
 
-
-} // namespace zeep::http
\ No newline at end of file
+} // namespace zeep::http
