@@ -117,7 +117,7 @@ void from_element(const E& e, Enum &en)
 // arrays
 
 // nice trick to enforce order in template selection
-template<unsigned N> struct priority_tag : priority_tag < N - 1 > {};
+template<unsigned N> struct priority_tag /** @cond */ : priority_tag < N - 1 > /** @endcond */ {};
 template<> struct priority_tag<0> {};
 
 template<typename E>
