@@ -9,6 +9,7 @@
 /// This file contains definitions of various utility routines
 
 #include <zeep/config.hpp>
+#include <zeep/http/uri.hpp>
 
 #include <filesystem>
 
@@ -31,4 +32,21 @@ namespace zeep::http
 /// \return				True in case of a match
 bool glob_match(const std::filesystem::path& p, std::string pattern);
 
+/// \brief compare the path part of a uri with a glob pattern
+///
+/// Returns true if the path \a p matches \a pattern
+/// Matching is done using shell like glob patterns:
+///
+/// construct     | Matches
+/// --------------|--------
+/// ?             | single character
+/// *             | zero or multiple characters
+/// {a,b}         | matching either pattern a or b
+///
+/// \param u			The uri whose path to match
+/// \param pattern		The pattern to match against
+/// \return				True in case of a match
+bool glob_match(const uri& u, std::string pattern);
+
+
 }
\ No newline at end of file
