@@ -1,16 +1,16 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
 #include <zeep/config.hpp>
 
+#include <iostream>
 #include <cerrno>
+#include <chrono>
 #include <functional>
 
-#include <boost/date_time/local_time/local_time.hpp>
-
 #if HTTP_SERVER_HAS_PREFORK
 
 #include <sys/wait.h>
@@ -26,9 +26,9 @@
 namespace zeep::http
 {
 
-bool read_socket_from_parent(int fd_socket, boost::asio::ip::tcp::socket &socket)
+bool read_socket_from_parent(int fd_socket, asio_ns::ip::tcp::socket &socket)
 {
-	using native_handle_type = boost::asio::ip::tcp::socket::native_handle_type;
+	using native_handle_type = asio_ns::ip::tcp::socket::native_handle_type;
 
 #if __APPLE__
 	// macos is special...
@@ -51,7 +51,7 @@ bool read_socket_from_parent(int fd_socket, boost::asio::ip::tcp::socket &socket
 	msg.msg_name = nullptr;
 	msg.msg_namelen = 0;
 
-	boost::asio::ip::tcp::socket::endpoint_type peer_endpoint;
+	asio_ns::ip::tcp::socket::endpoint_type peer_endpoint;
 
 	struct iovec iov[1];
 	iov[0].iov_base = peer_endpoint.data();
@@ -69,9 +69,9 @@ bool read_socket_from_parent(int fd_socket, boost::asio::ip::tcp::socket &socket
 		if (cmptr != nullptr and cmptr->cmsg_len == CMSG_LEN(sizeof(int)))
 		{
 			if (cmptr->cmsg_level != SOL_SOCKET)
-				std::cerr << "control level != SOL_SOCKET" << std::endl;
+				std::clog << "control level != SOL_SOCKET\n";
 			else if (cmptr->cmsg_type != SCM_RIGHTS)
-				std::cerr << "control type != SCM_RIGHTS";
+				std::clog << "control type != SCM_RIGHTS";
 			else
 			{
 				/* Produces warning: dereferencing type-punned pointer will break strict-aliasing rules [-Wstrict-aliasing]
@@ -94,7 +94,7 @@ bool read_socket_from_parent(int fd_socket, boost::asio::ip::tcp::socket &socket
 class child_process
 {
   public:
-	child_process(std::function<basic_server *(void)> constructor, boost::asio::io_context &io_context, boost::asio::ip::tcp::acceptor &acceptor, int nr_of_threads)
+	child_process(std::function<basic_server *(void)> constructor, asio_ns::io_context &io_context, asio_ns::ip::tcp::acceptor &acceptor, int nr_of_threads)
 		: m_constructor(constructor)
 		, m_acceptor(acceptor)
 		, m_socket(io_context)
@@ -109,33 +109,32 @@ class child_process
 		{
 			kill(m_pid, SIGKILL);
 
-			using namespace boost::posix_time;
-			auto now = second_clock::local_time();
+			const std::time_t now_t = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
 
 			int status;
 			if (waitpid(m_pid, &status, WUNTRACED | WCONTINUED) != -1)
 			{
-				if (WIFSIGNALED(status))
-					std::cerr << now << " child " << m_pid << " terminated by signal " << WTERMSIG(status) << std::endl;
-				else
-					std::cerr << now << " child terminated normally" << std::endl;
+				if (WIFSIGNALED(status) and WTERMSIG(status) != SIGKILL)
+					std::clog << std::put_time(std::localtime(&now_t), "%F %T") << " child " << m_pid << " terminated by signal " << WTERMSIG(status) << '\n';
+				// else
+				// 	std::clog << std::put_time(std::localtime(&now_t), "%F %T") << " child terminated normally\n";
 			}
 			else
-				std::cerr << now << "error in waitpid: " << strerror(errno) << std::endl;
+				std::clog << std::put_time(std::localtime(&now_t), "%F %T") << "error in waitpid: " << strerror(errno) << '\n';
 		}
 	}
 
 	void stop();
 
   private:
-	void handle_accept(const boost::system::error_code &ec);
+	void handle_accept(const asio_system_ns::error_code &ec);
 
 	void start();
 	void run();
 
 	std::function<basic_server *(void)> m_constructor;
-	boost::asio::ip::tcp::acceptor &m_acceptor;
-	boost::asio::ip::tcp::socket m_socket;
+	asio_ns::ip::tcp::acceptor &m_acceptor;
+	asio_ns::ip::tcp::socket m_socket;
 	int m_nr_of_threads;
 
 	int m_pid = -1;
@@ -188,7 +187,7 @@ void child_process::start()
 		}
 		catch (std::exception &e)
 		{
-			std::cerr << "Exception caught: " << e.what() << std::endl;
+			std::clog << "Exception caught: " << e.what() << '\n';
 			exit(1);
 		}
 
@@ -239,11 +238,11 @@ void child_process::stop()
 	}
 }
 
-void child_process::handle_accept(const boost::system::error_code &ec)
+void child_process::handle_accept(const asio_system_ns::error_code &ec)
 {
 	if (ec)
 	{
-		std::cerr << "Accept failed: " << ec << std::endl;
+		std::clog << "Accept failed: " << ec << '\n';
 		return;
 	}
 
@@ -252,7 +251,7 @@ void child_process::handle_accept(const boost::system::error_code &ec)
 		if (m_pid == -1 or m_fd == -1)
 			start();
 
-		using native_handle_type = boost::asio::ip::tcp::socket::native_handle_type;
+		using native_handle_type = asio_ns::ip::tcp::socket::native_handle_type;
 
 		struct msghdr msg;
 		union
@@ -290,14 +289,14 @@ void child_process::handle_accept(const boost::system::error_code &ec)
 		int err = sendmsg(m_fd, &msg, 0);
 		if (err < 0)
 		{
-			std::cerr << "Error passing file descriptor: " << strerror(errno) << std::endl;
+			std::clog << "Error passing file descriptor: " << strerror(errno) << '\n';
 			// ec = std::make_error_code(errno);
 			stop();
 		}
 	}
 	catch (const std::exception &e)
 	{
-		std::cerr << "error writing socket to client: " << e.what() << std::endl;
+		std::clog << "error writing socket to client: " << e.what() << '\n';
 
 		reply r = reply::stock_reply(service_unavailable);
 
@@ -305,11 +304,11 @@ void child_process::handle_accept(const boost::system::error_code &ec)
 
 		try
 		{
-			boost::asio::write(m_socket, buffers);
+			asio_ns::write(m_socket, buffers);
 		}
 		catch (const std::exception &e)
 		{
-			std::cerr << e.what() << std::endl;
+			std::clog << e.what() << '\n';
 		}
 	}
 
@@ -335,23 +334,26 @@ void preforked_server::run(const std::string &address, short port, int nr_of_pro
 	// first wait until we are allowed to start listening
 	std::unique_lock<std::mutex> lock(m_lock);
 
-	boost::asio::ip::tcp::acceptor acceptor(m_io_context);
+	asio_ns::ip::tcp::acceptor acceptor(m_io_context);
 
 	// then bind the address here
-	boost::asio::ip::tcp::endpoint endpoint;
+	asio_ns::ip::tcp::endpoint endpoint;
 	try
 	{
-		endpoint = boost::asio::ip::tcp::endpoint(boost::asio::ip::make_address(address), port);
+		endpoint = asio_ns::ip::tcp::endpoint(asio_ns::ip::make_address(address), port);
 	}
 	catch (const std::exception &e)
 	{
-		boost::asio::ip::tcp::resolver resolver(m_io_context);
-		boost::asio::ip::tcp::resolver::query query(address, std::to_string(port));
-		endpoint = *resolver.resolve(query);
+		asio_ns::ip::tcp::resolver resolver(m_io_context);
+		for (auto &ep : resolver.resolve(address, std::to_string(port)))
+		{
+			endpoint = ep;
+			break;
+		}
 	}
 
 	acceptor.open(endpoint.protocol());
-	acceptor.set_option(boost::asio::ip::tcp::acceptor::reuse_address(true));
+	acceptor.set_option(asio_ns::ip::tcp::acceptor::reuse_address(true));
 	acceptor.bind(endpoint);
 	acceptor.listen();
 
@@ -361,7 +363,7 @@ void preforked_server::run(const std::string &address, short port, int nr_of_pro
 	{
 		threads.emplace_back([this, nr_of_threads, &acceptor]()
 			{
-			auto work = boost::asio::make_work_guard(m_io_context);
+			auto work = asio_ns::make_work_guard(m_io_context);
 
 			child_process p(m_constructor, m_io_context, acceptor, nr_of_threads);
 			
