@@ -1,11 +1,11 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/date_time/local_time/local_time.hpp>
-
+#include <chrono>
+#include <iostream>
 #include <numeric>
 
 #include <zeep/http/reply.hpp>
@@ -39,6 +39,7 @@ namespace detail
 		{ forbidden, "Forbidden" },
 		{ not_found, "Not Found" },
 		{ method_not_allowed, "Method not allowed" },
+		{ unprocessable_entity, "Unprocessable Entity" },
 		{ internal_server_error, "Internal Server Error" },
 		{ not_implemented, "Not Implemented" },
 		{ bad_gateway, "Bad Gateway" },
@@ -54,6 +55,7 @@ namespace detail
 		{ proxy_authentication_required, "You are not authorized to use this proxy" },
 		{ forbidden, "Access to this location is forbidden" },
 		{ not_found, "The requested web page was not found on this server." },
+		{ unprocessable_entity, "Your request could not be handled since a parameter contained an invalid value" },
 		{ internal_server_error, "An internal error prevented the server from processing your request" },
 		{ not_implemented, "Your request could not be handled since the required code is not implemented" },
 		{ bad_gateway, "The server, while acting as a gateway or proxy, received an invalid response from the upstream server it accessed in attempting to fulfill the request. " },
@@ -116,16 +118,11 @@ reply::reply(status_type status, std::tuple<int, int> version)
 	, m_version_minor(std::get<1>(version))
 	, m_data(nullptr)
 {
-	using namespace boost::local_time;
-	using namespace boost::posix_time;
-
-	local_date_time t(local_sec_clock::local_time(time_zone_ptr()));
-	local_time_facet *lf(new local_time_facet("%a, %d %b %Y %H:%M:%S GMT"));
+	using namespace date;
+	using namespace std::chrono;
 
 	std::stringstream s;
-	s.imbue(std::locale(std::locale(), lf));
-
-	s << t;
+	to_stream(s, "%a, %d %b %Y %H:%M:%S GMT", system_clock::now());
 
 	set_header("Date", s.str());
 	set_header("Server", "libzeep");
@@ -198,13 +195,13 @@ reply &reply::operator=(reply &&rhs)
 		m_version_major = rhs.m_version_major;
 		m_version_minor = rhs.m_version_minor;
 		m_status = rhs.m_status;
-		m_data = rhs.m_data; rhs.m_data = nullptr;
-		m_headers = rhs.m_headers;
-		m_content = rhs.m_content;
+		m_data = std::exchange(rhs.m_data, nullptr);
+		m_headers = std::move(rhs.m_headers);
+		m_content = std::move(rhs.m_content);
 		m_chunked = rhs.m_chunked;
 
 		memcpy(m_size_buffer, rhs.m_size_buffer, sizeof(m_size_buffer));
-		m_status_line = rhs.m_status_line;
+		m_status_line = std::move(rhs.m_status_line);
 	}
 
 	return *this;
@@ -250,7 +247,7 @@ void reply::set_version(int version_major, int version_minor)
 			m_data->rdbuf()->pubseekoff(pos, std::ios_base::beg);
 		}
 
-		set_header("Content-Length", boost::lexical_cast<std::string>(length));
+		set_header("Content-Length", std::to_string(length));
 		remove_header("Transfer-Encoding");
 	}
 }
@@ -310,16 +307,11 @@ void reply::set_cookie(const char *name, const std::string &value, std::initiali
 
 void reply::set_delete_cookie(const char *name)
 {
-	using namespace boost::local_time;
-	using namespace boost::posix_time;
-
-	local_date_time t(local_sec_clock::local_time(time_zone_ptr()));
-	local_time_facet *lf(new local_time_facet("%a, %d %b %Y %H:%M:%S GMT"));
+    using namespace std::literals;
 
 	std::stringstream s;
-	s.imbue(std::locale(std::locale(), lf));
-
-	s << (t - hours(24));
+	const std::time_t now_t = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now() - 24h);
+	s << std::put_time(std::localtime(&now_t), "%a, %d %b %Y %H:%M:%S GMT");
 
 	set_cookie(name, "", {{"Expires", '"' + s.str() + '"'}});
 }
@@ -441,48 +433,56 @@ void reply::set_content(std::istream *idata, const std::string &contentType)
 	remove_header("Content-Length");
 }
 
-std::vector<boost::asio::const_buffer> reply::to_buffers() const
+std::vector<asio_ns::const_buffer> reply::to_buffers() const
 {
-	std::vector<boost::asio::const_buffer> result;
+	std::vector<asio_ns::const_buffer> result;
 
 	m_status_line =
 		"HTTP/" + std::to_string(m_version_major) + '.' + std::to_string(m_version_minor) + ' ' + std::to_string(m_status) + ' ' + get_status_text(m_status) + kCRLF;
 
-	result.push_back(boost::asio::buffer(m_status_line));
+	result.push_back(asio_ns::buffer(m_status_line));
 
 	for (const header &h : m_headers)
 	{
-		result.push_back(boost::asio::buffer(h.name));
-		result.push_back(boost::asio::buffer(kNameValueSeparator));
-		result.push_back(boost::asio::buffer(h.value));
-		result.push_back(boost::asio::buffer(kCRLF));
+		result.push_back(asio_ns::buffer(h.name));
+		result.push_back(asio_ns::buffer(kNameValueSeparator));
+		result.push_back(asio_ns::buffer(h.value));
+		result.push_back(asio_ns::buffer(kCRLF));
 	}
 
-	result.push_back(boost::asio::buffer(kCRLF));
-	result.push_back(boost::asio::buffer(m_content));
+	result.push_back(asio_ns::buffer(kCRLF));
+	result.push_back(asio_ns::buffer(m_content));
 
 	return result;
 }
 
-std::vector<boost::asio::const_buffer> reply::data_to_buffers()
+std::vector<asio_ns::const_buffer> reply::data_to_buffers()
 {
-	std::vector<boost::asio::const_buffer> result;
+	std::vector<asio_ns::const_buffer> result;
 
 	if (m_data != nullptr)
 	{
 		const unsigned int kMaxChunkSize = 10240;
 
 		m_buffer.resize(kMaxChunkSize);
-		std::streamsize n = m_data->rdbuf()->sgetn(m_buffer.data(), m_buffer.size());
+		std::streamsize n = 0;
+		try
+		{
+			n = m_data->rdbuf()->sgetn(m_buffer.data(), m_buffer.size());
+		}
+		catch (...)
+		{
+			std::clog << "Exception in reading from file\n";
+		}
 
 		// chunked encoding?
 		if (m_chunked)
 		{
 			if (n == 0)
 			{
-				result.push_back(boost::asio::buffer(kZERO));
-				result.push_back(boost::asio::buffer(kCRLF));
-				result.push_back(boost::asio::buffer(kCRLF));
+				result.push_back(asio_ns::buffer(kZERO));
+				result.push_back(asio_ns::buffer(kCRLF));
+				result.push_back(asio_ns::buffer(kCRLF));
 				delete m_data;
 				m_data = nullptr;
 			}
@@ -499,16 +499,16 @@ std::vector<boost::asio::const_buffer> reply::data_to_buffers()
 					n >>= 4;
 				}
 
-				result.push_back(boost::asio::buffer(p, e - p));
-				result.push_back(boost::asio::buffer(kCRLF));
-				result.push_back(boost::asio::buffer(&m_buffer[0], l));
-				result.push_back(boost::asio::buffer(kCRLF));
+				result.push_back(asio_ns::buffer(p, e - p));
+				result.push_back(asio_ns::buffer(kCRLF));
+				result.push_back(asio_ns::buffer(&m_buffer[0], l));
+				result.push_back(asio_ns::buffer(kCRLF));
 			}
 		}
 		else
 		{
 			if (n > 0)
-				result.push_back(boost::asio::buffer(&m_buffer[0], n));
+				result.push_back(asio_ns::buffer(&m_buffer[0], n));
 			else
 			{
 				delete m_data;
@@ -528,9 +528,9 @@ reply reply::stock_reply(status_type status, const std::string &info)
 	{
 		std::stringstream text;
 
-		text << "<html>" << std::endl
-			 << "  <body>" << std::endl
-			 << "    <h1>" << get_status_text(status) << "</h1>" << std::endl;
+		text << "<html>\n"
+			 << "  <body>\n"
+			 << "    <h1>" << get_status_text(status) << "</h1>\n";
 
 		if (not info.empty())
 		{
@@ -553,10 +553,10 @@ reply reply::stock_reply(status_type status, const std::string &info)
 				}
 			}
 
-			text << "</p>" << std::endl;
+			text << "</p>\n";
 		}
 
-		text << "  </body>" << std::endl
+		text << "  </body>\n"
 			 << "</html>";
 		result.set_content(text.str(), "text/html; charset=utf-8");
 	}
@@ -571,7 +571,7 @@ reply reply::stock_reply(status_type status)
 	return stock_reply(status, "");
 }
 
-reply reply::redirect(const std::string &location, status_type status)
+reply reply::redirect(const uri &location, status_type status)
 {
 	reply result;
 
@@ -582,18 +582,14 @@ reply reply::redirect(const std::string &location, status_type status)
 		"<html><head><title>" + text + "</title></head><body><h1>" +
  		std::to_string(status) + ' ' + text + "</h1></body></html>";
 
-	// Check if the location is a valid URL
-	if (not is_valid_uri(location))
-		throw exception("Invalid redirect location");
-
-	result.set_header("Location", location);
+	result.set_header("Location", location.string());
 	result.set_header("Content-Length", std::to_string(result.m_content.length()));
 	result.set_header("Content-Type", "text/html; charset=utf-8");
 
 	return result;
 }
 
-reply reply::redirect(const std::string &location)
+reply reply::redirect(const uri &location)
 {
 	return redirect(location, moved_temporarily);
 }
@@ -601,7 +597,7 @@ reply reply::redirect(const std::string &location)
 size_t reply::size() const
 {
 	auto buffers = to_buffers();
-	return std::accumulate(buffers.begin(), buffers.end(), 0LL, [](size_t m, auto &buffer) { return m + boost::asio::buffer_size(buffer); });
+	return std::accumulate(buffers.begin(), buffers.end(), 0LL, [](size_t m, auto &buffer) { return m + asio_ns::buffer_size(buffer); });
 }
 
 std::ostream &operator<<(std::ostream &lhs, const reply &rhs)
