@@ -1,12 +1,11 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
 #include <zeep/crypto.hpp>
 #include <zeep/http/rest-controller.hpp>
-#include <zeep/http/uri.hpp>
 
 namespace zeep::http
 {
@@ -19,7 +18,7 @@ rest_controller::~rest_controller()
 
 bool rest_controller::handle_request(http::request& req, http::reply& rep)
 {
-	std::string p = get_prefixless_path(req);
+	auto p = get_prefixless_path(req).string();
 
     bool result = false;
 	for (auto& mp: m_mountpoints)
@@ -43,14 +42,17 @@ bool rest_controller::handle_request(http::request& req, http::reply& rep)
 			for (size_t i = 0; i < mp->m_path_params.size(); ++i)
 			{
 				std::string v = m[i + 1].str();
-				decode_url(v);
+				v = decode_url(v);
 				params.m_path_parameters.push_back({ mp->m_path_params[i], v });
 			}
 		}
 
 		try
 		{
-			mp->call(params, rep);
+			if (req.get_method() == "OPTIONS")
+				get_options(req, rep);
+			else
+				mp->call(params, rep);
 		}
 		catch (status_type s)
 		{
@@ -76,4 +78,4 @@ bool rest_controller::handle_request(http::request& req, http::reply& rep)
 	return result;
 }
 
-}
\ No newline at end of file
+}
