@@ -1,20 +1,17 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //   Distributed under the Boost Software License, Version 1.0.
 //      (See accompanying file LICENSE_1_0.txt or copy at
 //            http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/date_time/posix_time/posix_time.hpp>
-#include <boost/date_time/local_time/local_time.hpp>
+#include "glob.hpp"
 
 #include <zeep/crypto.hpp>
 #include <zeep/http/security.hpp>
-#include <zeep/json/parser.hpp>
 #include <zeep/http/uri.hpp>
+#include <zeep/json/parser.hpp>
 
-#include "glob.hpp"
-
-namespace pt = boost::posix_time;
+#include <iostream>
 
 namespace zeep::http
 {
@@ -22,21 +19,50 @@ namespace zeep::http
 namespace
 {
 #define BASE64URL "(?:[-_A-Za-z0-9]{4})*(?:[-_A-Za-z0-9]{2,3})?"
-std::regex kJWTRx("^(" BASE64URL R"()\.()" BASE64URL R"()\.()" BASE64URL ")$" );
+	std::regex kJWTRx("^(" BASE64URL R"()\.()" BASE64URL R"()\.()" BASE64URL ")$");
+} // namespace
+
+// --------------------------------------------------------------------
+
+bool user_service::user_is_valid(const json::element &credentials) const
+{
+	return user_is_valid(credentials["username"].as<std::string>());
+}
+
+bool user_service::user_is_valid(const std::string &username) const
+{
+	bool result = false;
+
+	try
+	{
+		auto user = load_user(username);
+		result = user.username == username;
+	}
+	catch (...)
+	{
+	}
+
+	return result;
 }
 
 // --------------------------------------------------------------------
 
-void security_context::validate_request(request& req) const
+security_context::security_context(const std::string &secret, user_service &users, bool defaultAccessAllowed)
+	: m_secret(secret)
+	, m_users(users)
+	, m_default_allow(defaultAccessAllowed)
+	, m_default_jwt_exp(date::years{ 1 })
+{
+	register_password_encoder<pbkdf2_sha256_password_encoder>();
+}
+
+void security_context::validate_request(request &req) const
 {
 	bool allow = m_default_allow;
 
 	for (;;)
 	{
-		std::string path = uri(req.get_uri()).get_path().generic_string();
-
-		if (path.front() != '/')
-			path.insert(path.begin(), '/');
+		auto path = req.get_uri();
 
 		std::set<std::string> roles;
 
@@ -49,7 +75,7 @@ void security_context::validate_request(request& req) const
 			std::smatch m;
 			if (not std::regex_match(access_token, m, kJWTRx))
 				break;
-			
+
 			json::element JOSEHeader;
 			json::parse_json(decode_base64url(m[1].str()), JOSEHeader);
 
@@ -66,12 +92,25 @@ void security_context::validate_request(request& req) const
 			json::element credentials;
 			json::parse_json(decode_base64url(m[2].str()), credentials);
 
+			// check exp
+			using namespace std::chrono;
+
+			auto exp = credentials["exp"].as<int64_t>();
+			auto exp_t = time_point<system_clock>() + seconds{ exp };
+
+			if (system_clock::now() > exp_t)
+				break; // expired
+
 			if (not credentials.is_object() or not credentials["role"].is_array())
 				break;
 
-			for (auto role: credentials["role"])
+			// make sure user still exists.
+			if (not m_users.user_is_valid(credentials))
+				break;
+
+			for (auto role : credentials["role"])
 				roles.insert(role.as<std::string>());
-			
+
 			req.set_credentials(std::move(credentials));
 
 			break;
@@ -80,11 +119,11 @@ void security_context::validate_request(request& req) const
 		// first check if this page is allowed without any credentials
 		// that means, the first rule that matches this uri should allow
 		// access.
-		for (auto& rule: m_rules)
+		for (auto &rule : m_rules)
 		{
 			if (not glob_match(path, rule.m_pattern))
 				continue;
-			
+
 			if (rule.m_roles.empty())
 				allow = true;
 			else
@@ -108,7 +147,7 @@ void security_context::validate_request(request& req) const
 		if (req_csrf_cookie != req_csrf_param)
 		{
 			allow = false;
-			std::cerr << "CSRF validation failed" << std::endl;
+			std::cerr << "CSRF validation failed\n";
 		}
 	}
 
@@ -118,64 +157,87 @@ void security_context::validate_request(request& req) const
 
 // --------------------------------------------------------------------
 
-void security_context::add_authorization_headers(reply &rep, const user_details user)
+void security_context::add_authorization_headers(reply &rep, const user_details user,
+	std::chrono::system_clock::duration exp)
 {
 	using namespace json::literals;
 
+	using namespace date;
+	using namespace std::chrono;
+
 	auto JOSEHeader = R"({
 		"typ": "JWT",
 		"alg": "HS256"
 	})"_json;
 
+	auto exp_t = duration_cast<seconds>(system_clock::now() + exp - system_clock::time_point()).count();
+
 	json::element credentials{
-		{ "username", user.username }
+		{ "username", user.username },
+		{ "exp", exp_t }
 	};
 
-	for (auto& role: user.roles)
+	for (auto &role : user.roles)
 		credentials["role"].push_back(role);
 
 	auto h1 = encode_base64url(JOSEHeader.as<std::string>());
 	auto h2 = encode_base64url(credentials.as<std::string>());
 	auto h3 = encode_base64url(hmac_sha256(h1 + '.' + h2, m_secret));
 
-	rep.set_cookie("access_token", h1 + '.' + h2 + '.' + h3, {
-		{ "HttpOnly", "" },
-		{ "SameSite", "Lax" }
-	});
-}
+	std::stringstream s;
+	const std::time_t now_t = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now() + exp);
+	s << std::put_time(std::localtime(&now_t), "%a, %d %b %Y %H:%M:%S GMT");
 
-// --------------------------------------------------------------------
-
-void security_context::verify_username_password(const std::string& username, const std::string& raw_password, reply &rep)
-{
-	try
-	{
-		auto user = m_users.load_user(username);
-		
-		bool match = false;
-		for (auto const& [name, pwenc]: m_known_password_encoders)
+	rep.set_cookie("access_token", h1 + '.' + h2 + '.' + h3,
+		// clang-format off
 		{
-			if (user.password.compare(0, name.length(), name) != 0)
-				continue;
-			
-			match = pwenc->matches(raw_password, user.password);
-			break;
+			{ "HttpOnly", "" },
+			{ "SameSite", "Lax" },
+			{ "Expires", '"' + s.str() + '"' }
 		}
+		// clang-format on
+	);
+}
 
-		if (not match)
-			throw invalid_password_exception();
+void security_context::add_authorization_headers(reply &rep, const user_details user)
+{
+	using namespace date;
+	using namespace std::chrono;
 
-		add_authorization_headers(rep, user);
-	}
-	catch (const std::exception &)
-	{
-		throw invalid_password_exception();
-	}
+	add_authorization_headers(rep, user, m_default_jwt_exp);
 }
 
 // --------------------------------------------------------------------
 
-std::pair<std::string,bool> security_context::get_csrf_token(request& req)
+bool security_context::verify_username_password(const std::string &username, const std::string &raw_password)
+{
+	bool result = false;
+
+	auto user = m_users.load_user(username);
+
+	for (auto const &[name, pwenc] : m_known_password_encoders)
+	{
+		if (user.password.compare(0, name.length(), name) != 0)
+			continue;
+
+		result = pwenc->matches(raw_password, user.password);
+		break;
+	}
+
+	return result;
+}
+
+void security_context::verify_username_password(const std::string &username, const std::string &raw_password, reply &rep)
+{
+	if (not verify_username_password(username, raw_password))
+		throw invalid_password_exception();
+
+	add_authorization_headers(rep, m_users.load_user(username));
+}
+
+// --------------------------------------------------------------------
+
+std::pair<std::string, bool> security_context::get_csrf_token(request &req)
 {
 	// See if we need to add a new csrf token
 	bool csrf_is_new = false;
@@ -189,4 +251,4 @@ std::pair<std::string,bool> security_context::get_csrf_token(request& req)
 	return { csrf, csrf_is_new };
 }
 
-}
\ No newline at end of file
+} // namespace zeep::http
\ No newline at end of file
