@@ -1,11 +1,11 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //   Distributed under the Boost Software License, Version 1.0.
 //      (See accompanying file LICENSE_1_0.txt or copy at
 //            http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/date_time/local_time/local_time.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
+#include <chrono>
+#include <iostream>
 
 #include <zeep/http/connection.hpp>
 #include <zeep/http/controller.hpp>
@@ -32,15 +32,10 @@ namespace detail
 basic_server::basic_server()
 	: m_log_forwarded(true)
 	, m_security_context(nullptr)
-	, m_allowed_methods{"GET", "POST", "PUT", "OPTIONS", "HEAD", "DELETE"}
+	, m_allowed_methods{ "GET", "POST", "PUT", "OPTIONS", "HEAD", "DELETE" }
 {
-	using namespace boost::local_time;
-
-	local_time_facet *lf(new local_time_facet("[%d/%b/%Y:%H:%M:%S %z]"));
-	std::cout.imbue(std::locale(std::cout.getloc(), lf));
-
 	// add a default error handler
-	add_error_handler(new error_handler());
+	add_error_handler(new default_error_handler());
 }
 
 basic_server::basic_server(security_context *s_cntxt)
@@ -70,32 +65,50 @@ void basic_server::bind(const std::string &address, unsigned short port)
 	m_address = address;
 	m_port = port;
 
-	m_acceptor.reset(new boost::asio::ip::tcp::acceptor(get_io_context()));
+	m_acceptor.reset(new asio_ns::ip::tcp::acceptor(get_io_context()));
 	m_new_connection.reset(new connection(get_io_context(), *this));
 
 	// then bind the address here
-	boost::asio::ip::tcp::endpoint endpoint;
+	asio_ns::ip::tcp::endpoint endpoint;
 
-	try
+	asio_system_ns::error_code ec;
+	auto addr = asio_ns::ip::make_address(address, ec);
+	if (not ec)
+		endpoint = asio_ns::ip::tcp::endpoint(addr, port);
+	else
 	{
-		endpoint = boost::asio::ip::tcp::endpoint(boost::asio::ip::make_address(address), port);
-	}
-	catch (const std::exception &e)
-	{
-		boost::asio::ip::tcp::resolver resolver(get_io_context());
-		boost::asio::ip::tcp::resolver::query query(address, std::to_string(port));
-		endpoint = *resolver.resolve(query);
+		asio_ns::ip::tcp::resolver resolver(get_io_context());
+		for (auto &ep : resolver.resolve(address, std::to_string(port)))
+		{
+			endpoint = ep;
+			break;
+		}
 	}
 
 	m_acceptor->open(endpoint.protocol());
-	m_acceptor->set_option(boost::asio::ip::tcp::acceptor::reuse_address(true));
+	m_acceptor->set_option(asio_ns::ip::tcp::acceptor::reuse_address(true));
 	m_acceptor->bind(endpoint);
 	m_acceptor->listen();
 	m_acceptor->async_accept(m_new_connection->get_socket(),
-		[this](boost::system::error_code ec)
+		[this](asio_system_ns::error_code ec)
 		{ this->handle_accept(ec); });
 }
 
+void basic_server::get_options_for_request(const request &req, reply &rep)
+{
+	rep = reply::stock_reply(no_content);
+	rep.set_header("Allow", join(m_allowed_methods, ","));
+	rep.set_header("Cache-Control", "max-age=86400");
+
+	set_access_control_headers(req, rep);
+}
+
+void basic_server::set_access_control_headers([[maybe_unused]] const request &req, reply &rep)
+{
+	if (m_access_control)
+		m_access_control->get_access_control_headers(rep);
+}
+
 void basic_server::add_controller(controller *c)
 {
 	m_controllers.push_back(c);
@@ -111,7 +124,7 @@ void basic_server::add_error_handler(error_handler *eh)
 void basic_server::run(int nr_of_threads)
 {
 	// keep the server at work until we call stop
-	auto work = boost::asio::make_work_guard(get_io_context());
+	auto work = asio_ns::make_work_guard(get_io_context());
 
 	for (int i = 0; i < nr_of_threads; ++i)
 		m_threads.emplace_back([this]()
@@ -134,14 +147,14 @@ void basic_server::stop()
 	m_acceptor.reset();
 }
 
-void basic_server::handle_accept(boost::system::error_code ec)
+void basic_server::handle_accept(asio_system_ns::error_code ec)
 {
 	if (not ec)
 	{
 		m_new_connection->start();
 		m_new_connection.reset(new connection(get_io_context(), *this));
 		m_acceptor->async_accept(m_new_connection->get_socket(),
-			[this](boost::system::error_code ec)
+			[this](asio_system_ns::error_code ec)
 			{ this->handle_accept(ec); });
 	}
 }
@@ -153,16 +166,14 @@ std::ostream &basic_server::get_log()
 	return *detail::s_log;
 }
 
-void basic_server::handle_request(boost::asio::ip::tcp::socket &socket, request &req, reply &rep)
+void basic_server::handle_request(asio_ns::ip::tcp::socket &socket, request &req, reply &rep)
 {
-	using namespace boost::posix_time;
-
 	// we're pessimistic
 	rep = reply::stock_reply(not_found);
 
 	// set up a logging stream and collect logging information
 	detail::s_log.reset(new std::ostringstream);
-	ptime start = second_clock::local_time();
+	auto start = std::chrono::system_clock::now();
 
 	std::string referer("-"), userAgent("-"), accept, client;
 
@@ -193,8 +204,8 @@ void basic_server::handle_request(boost::asio::ip::tcp::socket &socket, request
 		// causing aborting exceptions, so I moved it here.
 		if (client.empty())
 		{
-			boost::asio::ip::address addr = socket.remote_endpoint().address();
-			client = boost::lexical_cast<std::string>(addr);
+			asio_ns::ip::address addr = socket.remote_endpoint().address();
+			client = addr.to_string();
 		}
 
 		req.set_remote_address(client);
@@ -213,23 +224,47 @@ void basic_server::handle_request(boost::asio::ip::tcp::socket &socket, request
 			std::tie(csrf, csrf_is_new) = m_security_context->get_csrf_token(req);
 		}
 
-		// parse the uri
-		std::string path = uri(req.get_uri()).get_path().generic_string();
-
 		// do the actual work.
+		bool processed = false;
 		for (auto c : m_controllers)
 		{
-			if (not c->path_matches_prefix(path))
+			if (not c->path_matches_prefix(req.get_uri()))
 				continue;
 
 			if (c->dispatch_request(socket, req, rep))
+			{
+				processed = true;
 				break;
+			}
 		}
 
-		if (method == "HEAD")
+		if (not processed)
+		{
+			for (auto eh : m_error_handlers)
+			{
+				try
+				{
+					if (eh->create_error_reply(req, not_found, rep))
+						break;
+				}
+				catch (...)
+				{
+				}
+			}
+		}
+
+		if (method == "HEAD" or method == "OPTIONS")
 			rep.set_content("", rep.get_content_type());
 		else if (csrf_is_new)
-			rep.set_cookie("csrf-token", csrf, {{"HttpOnly", ""}, {"SameSite", "Lax"}, {"Path", "/"}});
+			rep.set_cookie("csrf-token", csrf, { { "HttpOnly", "" }, { "SameSite", "Lax" }, { "Path", "/" } });
+
+		if (not m_context_name.empty() and
+			(rep.get_status() == moved_permanently or rep.get_status() == moved_temporarily))
+		{
+			auto location = rep.get_header("location");
+			if (location.front() == '/')
+				rep.set_header("location", m_context_name + location);
+		}
 
 		// work around buggy IE... also, using req.accept() doesn't work since it contains */* ... duh
 		if (starts_with(rep.get_content_type(), "application/xhtml+xml") and
@@ -238,6 +273,8 @@ void basic_server::handle_request(boost::asio::ip::tcp::socket &socket, request
 		{
 			rep.set_content_type("text/html; charset=utf-8");
 		}
+
+		set_access_control_headers(req, rep);
 	}
 	catch (...)
 	{
@@ -260,7 +297,7 @@ void basic_server::handle_request(boost::asio::ip::tcp::socket &socket, request
 
 void basic_server::log_request(const std::string &client,
 	const request &req, const reply &rep,
-	const boost::posix_time::ptime &start,
+	const std::chrono::system_clock::time_point &start,
 	const std::string &referer, const std::string &userAgent,
 	const std::string &entry) noexcept
 {
@@ -269,9 +306,7 @@ void basic_server::log_request(const std::string &client,
 		// protect the output stream from garbled log messages
 		std::unique_lock<std::mutex> lock(detail::s_log_lock);
 
-		using namespace boost::local_time;
-
-		local_date_time start_local(start, time_zone_ptr());
+		const std::time_t now_t = std::chrono::system_clock::to_time_t(start);
 
 		auto credentials = req.get_credentials();
 		std::string username = credentials.is_object() ? credentials["username"].as<std::string>() : "";
@@ -283,7 +318,7 @@ void basic_server::log_request(const std::string &client,
 		std::cout << client << ' '
 				  << "-" << ' '
 				  << username << ' '
-				  << start_local << ' '
+				  << std::put_time(std::localtime(&now_t), "[%d/%b/%Y:%H:%M:%S %z]") << ' '
 				  << '"' << req.get_method() << ' ' << req.get_uri() << ' '
 				  << "HTTP/" << major << '.' << minor << "\" "
 				  << rep.get_status() << ' '
@@ -292,9 +327,9 @@ void basic_server::log_request(const std::string &client,
 				  << '"' << userAgent << '"' << ' ';
 
 		if (entry.empty())
-			std::cout << '-' << std::endl;
+			std::cout << "-" << std::endl;
 		else
-			std::cout << '"' << entry << '"' << std::endl;
+			std::cout << std::quoted(entry) << std::endl;
 	}
 	catch (...)
 	{
