@@ -1,5 +1,5 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -37,7 +37,7 @@ xml::element make_envelope(xml::element&& data)
 		{ "soap:encodingStyle", "http://www.w3.org/2003/05/soap-encoding" }
 	});
 	auto& body = env.emplace_back("soap:Body");
-	body.emplace_back(std::forward<xml::element>(data));
+	body.emplace_back(std::move(data));
 	
 	return env;
 }
@@ -66,11 +66,9 @@ bool soap_controller::handle_request(request& req, reply& reply)
 {
 	bool result = false;
 
-	std::string p = uri(req.get_uri()).get_path().generic_string();
-	while (p.front() == '/')
-		p.erase(0, 1);
+	auto p = get_prefixless_path(req);
 	
-	if (req.get_method() == "POST" and p == m_prefix_path)
+	if (req.get_method() == "POST" and p.empty())
 	{
 		result = true;
 
@@ -110,7 +108,7 @@ bool soap_controller::handle_request(request& req, reply& reply)
 			reply.set_status(s);
 		}
 	}
-	else if (req.get_method() == "GET" and p == m_prefix_path + "/wsdl")
+	else if (req.get_method() == "GET" and p == "wsdl")
 	{
 		reply.set_content(make_wsdl());
 		reply.set_status(ok);
@@ -166,18 +164,18 @@ xml::element soap_controller::make_wsdl()
 		{ "name", m_service + "PortType" }
 	});
 	
-	// // // and the types
-	// // xml::type_map typeMap;
-	// // detail::message_map messageMap;
+	// and the types
+	xml::type_map typeMap;
+	message_map messageMap;
 	
-	// // for (auto& mp: m_mountpoints)
-	// // 	mp->collect(typeMap, messageMap, portType, binding);
+	for (auto& mp: m_mountpoints)
+		mp->describe(typeMap, messageMap, portType, binding);
 	
-	// // for (detail::message_map::iterator m = messageMap.begin(); m != messageMap.end(); ++m)
-	// // 	wsdl->append(m->second);
+	for (auto &m : messageMap)
+		wsdl.emplace_back(std::move(m.second));
 	
-	// // for (xml::type_map::iterator t = typeMap.begin(); t != typeMap.end(); ++t)
-	// // 	schema->append(t->second);
+	for (auto &t : typeMap)
+		schema.emplace_back(std::move(t.second));
 	
 	// finish with the wsdl:service
 	auto& service = wsdl.emplace_back("wsdl:service",
@@ -191,9 +189,11 @@ xml::element soap_controller::make_wsdl()
 		{ "binding", "ns:" + m_service }
 	});
 	
+	std::string location = get_context_name() + "/" + m_location;
+
 	port.emplace_back("soap:address",
 	{
-		{ "location", m_location }
+		{ "location", location }
 	});
 	
 	return wsdl;
