@@ -3,24 +3,27 @@
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
-#include <zeep/http/template-processor.hpp>
+#include <iostream>
+
 #include <zeep/http/tag-processor.hpp>
+#include <zeep/http/template-processor.hpp>
 
 namespace fs = std::filesystem;
-namespace pt = boost::posix_time;
 
 namespace zeep::http
 {
 
+#if ZEEP_SUPPORT_TAG_PROCESSOR_V1
+
 // --------------------------------------------------------------------
 //
 
-tag_processor_v1::tag_processor_v1(const char* ns)
-    : tag_processor(ns)
+tag_processor_v1::tag_processor_v1(const char *ns)
+	: tag_processor(ns)
 {
 }
 
-bool tag_processor_v1::process_el(const scope& scope, std::string& s)
+bool tag_processor_v1::process_el(const scope &scope, std::string &s)
 {
 	bool replaced = false;
 
@@ -31,7 +34,7 @@ bool tag_processor_v1::process_el(const scope& scope, std::string& s)
 		auto i = s.find('$', b);
 		if (i == std::string::npos)
 			break;
-		
+
 		char c2 = s[i + 1];
 		if (c2 != '{')
 		{
@@ -60,9 +63,9 @@ bool tag_processor_v1::process_el(const scope& scope, std::string& s)
 	return replaced;
 }
 
-void tag_processor_v1::process_xml(xml::node *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_xml(xml::node *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
-	xml::text* text = dynamic_cast<xml::text*>(node);
+	xml::text *text = dynamic_cast<xml::text *>(node);
 
 	if (text != nullptr)
 	{
@@ -74,14 +77,14 @@ void tag_processor_v1::process_xml(xml::node *node, const scope& scope, fs::path
 		return;
 	}
 
-	xml::element* e = dynamic_cast<xml::element*>(node);
+	xml::element *e = dynamic_cast<xml::element *>(node);
 	if (e == nullptr)
 		return;
 
 	// if node is one of our special nodes, we treat it here
 	if (e->get_ns() == m_ns)
 	{
-		xml::element* parent = e->parent();
+		xml::element *parent = e->parent();
 
 		try
 		{
@@ -89,7 +92,7 @@ void tag_processor_v1::process_xml(xml::node *node, const scope& scope, fs::path
 
 			process_tag(e->name(), e, scope, dir, loader);
 		}
-		catch (exception& ex)
+		catch (exception &ex)
 		{
 			parent->nodes().push_back(
 				xml::text("Error processing directive '" + e->get_qname() + "': " + ex.what()));
@@ -99,44 +102,55 @@ void tag_processor_v1::process_xml(xml::node *node, const scope& scope, fs::path
 		{
 			parent->erase(e);
 		}
-		catch (exception& ex)
+		catch (exception &ex)
 		{
-			std::cerr << "exception: " << ex.what() << std::endl
-					  << *e << std::endl;
+			std::clog << "exception: " << ex.what() << '\n'
+					  << *e << '\n';
 		}
 	}
 	else
 	{
-		for (auto& a : e->attributes())
+		for (auto &a : e->attributes())
 		{
 			std::string s = a.value();
 			if (process_el(scope, s))
 				a.value(s);
 		}
 
-		std::vector<xml::element*> nodes{ e->begin(), e->end() };
-		for (auto n: nodes)
+		std::vector<xml::element *> nodes{ e->begin(), e->end() };
+		for (auto n : nodes)
 			process_xml(n, scope, dir, loader);
 	}
 }
 
-void tag_processor_v1::process_tag(const std::string& tag, xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_tag(const std::string &tag, xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
-		 if (tag == "include")	process_include(node, scope, dir, loader);
-	else if (tag == "if")		process_if(node, scope, dir, loader);
-	else if (tag == "iterate")	process_iterate(node, scope, dir, loader);
-	else if (tag == "for")		process_for(node, scope, dir, loader);
-	else if (tag == "number")	process_number(node, scope, dir, loader);
-	else if (tag == "options")	process_options(node, scope, dir, loader);
-	else if (tag == "option")	process_option(node, scope, dir, loader);
-	else if (tag == "checkbox")	process_checkbox(node, scope, dir, loader);
+	if (tag == "include")
+		process_include(node, scope, dir, loader);
+	else if (tag == "if")
+		process_if(node, scope, dir, loader);
+	else if (tag == "iterate")
+		process_iterate(node, scope, dir, loader);
+	else if (tag == "for")
+		process_for(node, scope, dir, loader);
+	else if (tag == "number")
+		process_number(node, scope, dir, loader);
+	else if (tag == "options")
+		process_options(node, scope, dir, loader);
+	else if (tag == "option")
+		process_option(node, scope, dir, loader);
+	else if (tag == "checkbox")
+		process_checkbox(node, scope, dir, loader);
 	// else if (tag == "url")		process_url(node, scope, dir, loader);
-	else if (tag == "param")	process_param(node, scope, dir, loader);
-	else if (tag == "embed")	process_embed(node, scope, dir, loader);
-	else throw exception("unimplemented <m1:" + tag + "> tag");
+	else if (tag == "param")
+		process_param(node, scope, dir, loader);
+	else if (tag == "embed")
+		process_embed(node, scope, dir, loader);
+	else
+		throw exception("unimplemented <m1:" + tag + "> tag");
 }
 
-void tag_processor_v1::process_include(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_include(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	// an include directive, load file and include resulting content
 	std::string file = node->get_attribute("file");
@@ -156,7 +170,7 @@ void tag_processor_v1::process_include(xml::element *node, const scope& scope, f
 	parent->insert(node, std::move(doc.front()));
 }
 
-void tag_processor_v1::process_if(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_if(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	std::string test = node->get_attribute("test");
 	if (evaluate_el(scope, test))
@@ -164,7 +178,7 @@ void tag_processor_v1::process_if(xml::element *node, const scope& scope, fs::pa
 		auto parent = node->parent();
 		assert(parent);
 
-		for (auto& c: *node)
+		for (auto &c : *node)
 		{
 			auto copy = parent->emplace(node, std::move(c)); // insert before processing, to assign namespaces
 			process_xml(copy, scope, dir, loader);
@@ -172,7 +186,7 @@ void tag_processor_v1::process_if(xml::element *node, const scope& scope, fs::pa
 	}
 }
 
-void tag_processor_v1::process_iterate(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_iterate(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	using json::detail::value_type;
 
@@ -187,12 +201,12 @@ void tag_processor_v1::process_iterate(xml::element *node, const scope& scope, f
 	auto parent = node->parent();
 	assert(parent);
 
-	for (object& o : collection)
+	for (object &o : collection)
 	{
 		auto s(scope);
 		s.put(var, o);
 
-		for (auto& c: *node)
+		for (auto &c : *node)
 		{
 			auto i = parent->emplace(node, c); // insert before processing, to assign namespaces
 			process_xml(&*i, s, dir, loader);
@@ -200,7 +214,7 @@ void tag_processor_v1::process_iterate(xml::element *node, const scope& scope, f
 	}
 }
 
-void tag_processor_v1::process_for(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_for(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	object b = evaluate_el(scope, node->get_attribute("begin"));
 	object e = evaluate_el(scope, node->get_attribute("end"));
@@ -217,7 +231,7 @@ void tag_processor_v1::process_for(xml::element *node, const scope& scope, fs::p
 		auto parent = node->parent();
 		assert(parent);
 
-		for (auto& c: *node)
+		for (auto &c : *node)
 		{
 			auto i2 = parent->emplace(node, c); // insert before processing, to assign namespaces
 			process_xml(i2, s, dir, loader);
@@ -227,20 +241,20 @@ void tag_processor_v1::process_for(xml::element *node, const scope& scope, fs::p
 
 class with_thousands : public std::numpunct<char>
 {
-protected:
+  protected:
 	//	char_type do_thousands_sep() const	{ return tsp; }
 	std::string do_grouping() const { return "\03"; }
 	//	char_type do_decimal_point() const	{ return dsp; }
 };
 
-void tag_processor_v1::process_number(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_number(xml::element *node, const scope &scope, fs::path /*dir*/, basic_template_processor & /*loader*/)
 {
 	std::string number = node->get_attribute("n");
 	std::string format = node->get_attribute("f");
 
 	if (format == "#,##0B") // bytes, convert to a human readable form
 	{
-		const char kBase[] = {'B', 'K', 'M', 'G', 'T', 'P', 'E'}; // whatever
+		const char kBase[] = { 'B', 'K', 'M', 'G', 'T', 'P', 'E' }; // whatever
 
 		uint64_t nr = evaluate_el(scope, number).as<uint64_t>();
 		int base = 0;
@@ -276,7 +290,7 @@ void tag_processor_v1::process_number(xml::element *node, const scope& scope, fs
 	parent->nodes().emplace(node, zeep::xml::text(number));
 }
 
-void tag_processor_v1::process_options(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_options(xml::element *node, const scope &scope, fs::path /*dir*/, basic_template_processor & /*loader*/)
 {
 	using ::zeep::json::detail::value_type;
 
@@ -293,7 +307,7 @@ void tag_processor_v1::process_options(xml::element *node, const scope& scope, f
 		if (not selected.empty())
 			process_el(scope, selected);
 
-		for (object& o : collection)
+		for (object &o : collection)
 		{
 			zeep::xml::element option("option");
 
@@ -319,7 +333,7 @@ void tag_processor_v1::process_options(xml::element *node, const scope& scope, f
 	}
 }
 
-void tag_processor_v1::process_option(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_option(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	std::string value = node->get_attribute("value");
 	if (not value.empty())
@@ -339,14 +353,14 @@ void tag_processor_v1::process_option(xml::element *node, const scope& scope, fs
 	assert(parent);
 	parent->emplace(node, std::move(option));
 
-	for (auto& c: *node)
+	for (auto &c : *node)
 	{
 		auto i = option.emplace(option.end(), c);
 		process_xml(i, scope, dir, loader);
 	}
 }
 
-void tag_processor_v1::process_checkbox(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_checkbox(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	std::string name = node->get_attribute("name");
 	if (not name.empty())
@@ -367,53 +381,53 @@ void tag_processor_v1::process_checkbox(xml::element *node, const scope& scope,
 	assert(parent);
 	parent->emplace(node, std::move(checkbox));
 
-	for (auto& c: *node)
+	for (auto &c : *node)
 	{
 		auto i = checkbox.emplace(checkbox.end(), c);
 		process_xml(i, scope, dir, loader);
 	}
 }
 
-void tag_processor_v1::process_url(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
-{
-	// std::string var = node->get_attr("var");
+// void tag_processor_v1::process_url(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+// {
+// 	std::string var = node->get_attr("var");
 
-	// parameter_map parameters;
-	// get_parameters(scope, parameters);
+// 	parameter_map parameters;
+// 	get_parameters(scope, parameters);
 
-	// for (zeep::xml::element *e : *node)
-	// {
-	// 	if (e->ns() == m_ns and e->name() == "param")
-	// 	{
-	// 		std::string name = e->get_attr("name");
-	// 		std::string value = e->get_attr("value");
+// 	for (zeep::xml::element *e : *node)
+// 	{
+// 		if (e->ns() == m_ns and e->name() == "param")
+// 		{
+// 			std::string name = e->get_attr("name");
+// 			std::string value = e->get_attr("value");
 
-	// 		process_el(scope, value);
-	// 		parameters.replace(name, value);
-	// 	}
-	// }
+// 			process_el(scope, value);
+// 			parameters.replace(name, value);
+// 		}
+// 	}
 
-	// std::string url = scope["baseuri"].as<std::string>();
+// 	std::string url = scope["baseuri"].as<std::string>();
 
-	// bool first = true;
-	// for (parameter_map::value_type p : parameters)
-	// {
-	// 	url += (first ? '?' : '&');
-	// 	first = false;
+// 	bool first = true;
+// 	for (parameter_map::value_type p : parameters)
+// 	{
+// 		url += (first ? '?' : '&');
+// 		first = false;
 
-	// 	url += zeep::http::encode_url(p.first) + '=' + zeep::http::encode_url(p.second.as<std::string>());
-	// }
+// 		url += zeep::http::encode_url(p.first) + '=' + zeep::http::encode_url(p.second.as<std::string>());
+// 	}
 
-	// scope& s(const_cast<scope& >(scope));
-	// s.put(var, url);
-}
+// 	scope& s(const_cast<scope& >(scope));
+// 	s.put(var, url);
+// }
 
-void tag_processor_v1::process_param(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_param(xml::element * /*node*/, const scope & /*scope*/, fs::path /*dir*/, basic_template_processor & /*loader*/)
 {
 	throw exception("Invalid XML, cannot have a stand-alone mrs:param element");
 }
 
-void tag_processor_v1::process_embed(xml::element *node, const scope& scope, fs::path dir, basic_template_processor& loader)
+void tag_processor_v1::process_embed(xml::element *node, const scope &scope, fs::path dir, basic_template_processor &loader)
 {
 	// an embed directive, load xml from attribute and include parsed content
 	std::string xml = scope[node->get_attribute("var")].as<std::string>();
@@ -423,7 +437,7 @@ void tag_processor_v1::process_embed(xml::element *node, const scope& scope, fs:
 
 	zeep::xml::document doc;
 	doc.set_preserve_cdata(true);
-	
+
 	std::istringstream os(xml);
 	os >> doc;
 
@@ -433,6 +447,7 @@ void tag_processor_v1::process_embed(xml::element *node, const scope& scope, fs:
 	process_xml(i, scope, dir, loader);
 }
 
-// --------------------------------------------------------------------
+#endif
 
-}
+// --------------------------------------------------------------------
+} // namespace zeep::http
