@@ -1,10 +1,11 @@
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 //   Distributed under the Boost Software License, Version 1.0.
 //      (See accompanying file LICENSE_1_0.txt or copy at
 //            http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/date_time/posix_time/posix_time.hpp>
-#include <boost/date_time/local_time/local_time.hpp>
+#include <chrono>
+#include <fstream>
+#include <iostream>
 
 #include <zeep/xml/xpath.hpp>
 #include <zeep/http/template-processor.hpp>
@@ -41,12 +42,19 @@ std::istream* file_loader::load_file(const std::string& file, std::error_code& e
 	if (p.has_root_path())
 		p = fs::relative(p, p.root_path());
 
-	std::ifstream* result = new std::ifstream(m_docroot / p, std::ios::binary);
-	if (not result->is_open())
-	{
-		delete result;
-		result = nullptr;
+	std::ifstream* result = nullptr;
+
+	if (not fs::is_regular_file(m_docroot / p))
 		ec = std::make_error_code(std::errc::no_such_file_or_directory);
+	else
+	{
+		result = new std::ifstream(m_docroot / p, std::ios::binary);
+		if (not result->is_open())
+		{
+			delete result;
+			result = nullptr;
+			ec = std::make_error_code(std::errc::no_such_file_or_directory);
+		}
 	}
 	
 	return result;	
@@ -57,9 +65,6 @@ std::istream* file_loader::load_file(const std::string& file, std::error_code& e
 
 void basic_template_processor::handle_file(const http::request& request, const scope& scope, http::reply& reply)
 {
-	using namespace boost::local_time;
-	using namespace boost::posix_time;
-
 	std::error_code ec;
 	auto ft = file_time(scope["baseuri"].as<std::string>(), ec);
 
@@ -69,36 +74,19 @@ void basic_template_processor::handle_file(const http::request& request, const s
 		return;
 	}
 
-	auto lastWriteTime = std::chrono::duration_cast<std::chrono::seconds>(ft - decltype(ft)::clock::time_point{}).count();
-
-	// // std::chrono::time_point<std::filesystem::__file_clock> tp(ft);
-	// ptime()
-
-	// std::chrono::system_clock::from_time_t()
-
-	// // auto lastWriteTime = decltype(ft)::clock::to_time_t(ft);
-
-	// boost::chrono::duration_cast
-
-	// std::chrono::file_clock;
-	// ptime fpt(ft);
-	// auto lastWriteTime = to_time_t()
+	using namespace std::chrono;
+    auto fileDate = time_point_cast<system_clock::duration>(ft - decltype(ft)::clock::now() + system_clock::now());
 
 	std::string ifModifiedSince;
 	for (const http::header& h : request.get_headers())
 	{
 		if (iequals(h.name, "If-Modified-Since"))
 		{
-			local_date_time modifiedSince(local_sec_clock::local_time(time_zone_ptr()));
+			std::istringstream ss { h.value };
+			std::tm tm;
+			ss >> std::get_time(&tm, "%a, %d %b %Y %H:%M:%S GMT");
 
-			local_time_input_facet *lif1(new local_time_input_facet("%a, %d %b %Y %H:%M:%S GMT"));
-
-			std::stringstream ss;
-			ss.imbue(std::locale(std::locale::classic(), lif1));
-			ss.str(h.value);
-			ss >> modifiedSince;
-
-			local_date_time fileDate(from_time_t(lastWriteTime), time_zone_ptr());
+			auto modifiedSince = system_clock::from_time_t(std::mktime(&tm));
 
 			if (fileDate <= modifiedSince)
 			{
@@ -119,17 +107,6 @@ void basic_template_processor::handle_file(const http::request& request, const s
 		return;
 	}
 
-	std::stringstream out;
-
-	for (;;)
-	{
-		char buffer[1024];
-		auto r = in->readsome(buffer, sizeof(buffer));
-		if (r <= 0)
-			break;
-		out.write(buffer, r);
-	}
-
 	std::string mimetype = "text/plain";
 
 	if (file.extension() == ".css")
@@ -148,18 +125,20 @@ void basic_template_processor::handle_file(const http::request& request, const s
 		mimetype = "application/xhtml+xml";
 	else if (file.extension() == ".ico")
 		mimetype = "image/x-icon";
+	else if (file.extension() == ".json" or file.extension() == ".schema")
+		mimetype = "application/json";
+	else if (file.extension() == ".bz2")
+		mimetype = "application/x-bzip2";
+	else if (file.extension() == ".gz")
+		mimetype = "application/gzip";
 
-	reply.set_content(out.str(), mimetype);
+	reply.set_content(in.release(), mimetype);
 
-	local_date_time t(local_sec_clock::local_time(time_zone_ptr()));
-	local_time_facet *lf(new local_time_facet("%a, %d %b %Y %H:%M:%S GMT"));
+	using namespace date;
+	using namespace std::chrono;
 
 	std::stringstream s;
-	s.imbue(std::locale(std::cout.getloc(), lf));
-
-	ptime pt = from_time_t(lastWriteTime);
-	local_date_time t2(pt, time_zone_ptr());
-	s << t2;
+	to_stream(s, "%a, %d %b %Y %H:%M:%S GMT", fileDate);
 
 	reply.set_header("Last-Modified", s.str());
 }
@@ -225,7 +204,7 @@ void basic_template_processor::load_template(const std::string& file, xml::docum
 
 	if (not data)
 	{
-#if defined(_MSC_VER)
+#if defined(_WIN32)
 		char msg[1024] = "";
 
 		DWORD dw = ::GetLastError();
@@ -261,7 +240,7 @@ void basic_template_processor::load_template(const std::string& file, xml::docum
 	}
 	catch (const std::exception& ex)
 	{
-		std::cerr << "Error parsing template: " << ex.what() << std::endl;
+		std::clog << "Error parsing template: " << ex.what() << '\n';
 		throw;
 	}
 
@@ -328,7 +307,7 @@ void basic_template_processor::create_reply_from_template(const std::string& fil
 	reply.set_content(doc);
 }
 
-void basic_template_processor::init_scope(scope& scope)
+void basic_template_processor::init_scope(scope& /*scope*/)
 {
 }
 
