@@ -1,15 +1,16 @@
 // Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
-//        Copyright Maarten L. Hekkelman, 2014-2022
+//        Copyright Maarten L. Hekkelman, 2014-2023
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
-#include <set>
-#include <stack>
-
 #include <zeep/xml/document.hpp>
 #include <zeep/xml/xpath.hpp>
 
+#include <algorithm>
+#include <set>
+#include <stack>
+
 namespace zeep::xml
 {
 
@@ -19,7 +20,7 @@ const std::set<std::string> kEmptyHTMLElements{
 
 // --------------------------------------------------------------------
 
-void write_string(std::ostream& os, const std::string& s, bool escape_whitespace, bool escape_quot, bool trim, float version)
+void write_string(std::ostream &os, const std::string &s, bool escape_whitespace, bool escape_quot, bool trim, float version)
 {
 	bool last_is_space = false;
 
@@ -35,22 +36,60 @@ void write_string(std::ostream& os, const std::string& s, bool escape_whitespace
 
 		switch (c)
 		{
-			case '&':	os << "&amp;";			last_is_space = false; break;
-			case '<':	os << "&lt;";			last_is_space = false; break;
-			case '>':	os << "&gt;";			last_is_space = false; break;
-			case '\"':	if (escape_quot)		os << "&quot;"; else os << static_cast<char>(c); last_is_space = false; break;
-			case '\n':	if (escape_whitespace)	os << "&#10;"; else os << static_cast<char>(c); last_is_space = true; break;
-			case '\r':	if (escape_whitespace)	os << "&#13;"; else os << static_cast<char>(c); last_is_space = false; break;
-			case '\t':	if (escape_whitespace)	os << "&#9;"; else os << static_cast<char>(c); last_is_space = false; break;
-			case ' ':	if (not trim or not last_is_space) os << ' '; last_is_space = true; break;
-			case 0:		throw exception("Invalid null character in XML content");
-			default:	if (c >= 0x0A0 or (version == 1.0 ? is_valid_xml_1_0_char(c) : is_valid_xml_1_1_char(c)))
-							for (auto ci = sb; ci < sp; ++ci)
-								os << *ci;
-						else
-							os << "&#" << static_cast<int>(c) << ';';
-						last_is_space = false;
-						break;
+			case '&':
+				os << "&amp;";
+				last_is_space = false;
+				break;
+			case '<':
+				os << "&lt;";
+				last_is_space = false;
+				break;
+			case '>':
+				os << "&gt;";
+				last_is_space = false;
+				break;
+			case '\"':
+				if (escape_quot)
+					os << "&quot;";
+				else
+					os << static_cast<char>(c);
+				last_is_space = false;
+				break;
+			case '\n':
+				if (escape_whitespace)
+					os << "&#10;";
+				else
+					os << static_cast<char>(c);
+				last_is_space = true;
+				break;
+			case '\r':
+				if (escape_whitespace)
+					os << "&#13;";
+				else
+					os << static_cast<char>(c);
+				last_is_space = false;
+				break;
+			case '\t':
+				if (escape_whitespace)
+					os << "&#9;";
+				else
+					os << static_cast<char>(c);
+				last_is_space = false;
+				break;
+			case ' ':
+				if (not trim or not last_is_space)
+					os << ' ';
+				last_is_space = true;
+				break;
+			case 0: throw exception("Invalid null character in XML content");
+			default:
+				if (c >= 0x0A0 or (version == 1.0 ? is_valid_xml_1_0_char(c) : is_valid_xml_1_1_char(c)))
+					for (auto ci = sb; ci < sp; ++ci)
+						os << *ci;
+				else
+					os << "&#" << static_cast<int>(c) << ';';
+				last_is_space = false;
+				break;
 		}
 
 		sb = sp;
@@ -64,30 +103,30 @@ node::~node()
 	// avoid deep recursion and stack overflows
 	while (m_next != nullptr)
 	{
-		node* n = m_next;
+		node *n = m_next;
 		m_next = n->m_next;
 		n->m_next = nullptr;
 		delete n;
 	}
 }
 
-element* node::root()
+element *node::root()
 {
-	element* result = nullptr;
+	element *result = nullptr;
 	if (m_parent != nullptr)
 		result = m_parent->root();
 	return result;
 }
 
-const element* node::root() const
+const element *node::root() const
 {
-	const element* result = nullptr;
+	const element *result = nullptr;
 	if (m_parent != nullptr)
 		result = m_parent->root();
 	return result;
 }
 
-bool node::equals(const node* n) const
+bool node::equals(const node *n) const
 {
 	assert(false);
 	return n == this;
@@ -101,37 +140,37 @@ std::string node::lang() const
 	return result;
 }
 
-void node::insert_sibling(node* n, node* before)
+void node::insert_sibling(node *n, node *before)
 {
-	node* p = this;
+	node *p = this;
 	while (p->m_next != nullptr and p->m_next != before)
 		p = p->m_next;
 
 	if (p->m_next != before and before != nullptr)
 		throw zeep::exception("before argument in insert_sibling is not valid");
-	
+
 	p->m_next = n;
 	n->m_prev = p;
 	n->m_parent = m_parent;
 	n->m_next = before;
-	
+
 	if (before != nullptr)
 		before->m_prev = n;
 
-// #if DEBUG
-// validate();
-// n->validate();
-// if (before) before->validate();
-// #endif
+	// #if DEBUG
+	// validate();
+	// n->validate();
+	// if (before) before->validate();
+	// #endif
 }
 
-void node::remove_sibling(node* n)
+void node::remove_sibling(node *n)
 {
-	assert (this != n);
+	assert(this != n);
 	if (this == n)
 		throw exception("inconsistent node tree");
 
-	node* p = this;
+	node *p = this;
 	while (p != nullptr and p->m_next != n)
 		p = p->m_next;
 
@@ -146,7 +185,7 @@ void node::remove_sibling(node* n)
 		throw exception("remove for a node not found in the list");
 }
 
-void node::parent(element* n)
+void node::parent(element *n)
 {
 	assert(m_parent == nullptr);
 	m_parent = n;
@@ -185,7 +224,7 @@ std::string node::get_ns() const
 	return namespace_for_prefix(p);
 }
 
-std::string node::namespace_for_prefix(const std::string& prefix) const
+std::string node::namespace_for_prefix(const std::string &prefix) const
 {
 	std::string result;
 	if (m_parent != nullptr)
@@ -193,15 +232,15 @@ std::string node::namespace_for_prefix(const std::string& prefix) const
 	return result;
 }
 
-std::pair<std::string,bool> node::prefix_for_namespace(const std::string& uri) const
+std::pair<std::string, bool> node::prefix_for_namespace(const std::string &uri) const
 {
-	std::pair<std::string,bool> result{};
+	std::pair<std::string, bool> result{};
 	if (m_parent != nullptr)
 		result = m_parent->prefix_for_namespace(uri);
 	return result;
 }
 
-std::string node::prefix_tag(const std::string& tag, const std::string& uri) const
+std::string node::prefix_tag(const std::string &tag, const std::string &uri) const
 {
 	auto prefix = prefix_for_namespace(uri);
 	return prefix.second ? prefix.first + ':' + tag : tag;
@@ -209,15 +248,16 @@ std::string node::prefix_tag(const std::string& tag, const std::string& uri) con
 
 void node::validate()
 {
-	if (m_parent and dynamic_cast<element*>(this) != nullptr and
-			(std::find_if(m_parent->m_nodes.begin(), m_parent->m_nodes.end(), [this](auto& i) { return &i == this; }) == m_parent->m_nodes.end()))
+	if (m_parent and dynamic_cast<element *>(this) != nullptr and
+		(std::find_if(m_parent->m_nodes.begin(), m_parent->m_nodes.end(), [this](auto &i)
+			 { return &i == this; }) == m_parent->m_nodes.end()))
 		throw exception("validation error: parent does not know node");
 	if (m_next and m_next->m_prev != this)
 		throw exception("validation error: m_next->m_prev != this");
 	if (m_prev and m_prev->m_next != this)
 		throw exception("validation error: m_prev->m_next != this");
-	
-	node* n = this;
+
+	node *n = this;
 	while (n != nullptr and n->m_next != this)
 		n = n->m_next;
 	if (n == this)
@@ -228,7 +268,7 @@ void node::validate()
 		n = n->m_prev;
 	if (n == this)
 		throw exception("cycle in node list");
-	
+
 	if (m_next)
 		m_next->validate();
 }
@@ -236,38 +276,37 @@ void node::validate()
 // --------------------------------------------------------------------
 // comment
 
-bool comment::equals(const node* n) const
+bool comment::equals(const node *n) const
 {
-	return
-		dynamic_cast<const comment*>(n) != nullptr and
-		m_text == static_cast<const comment*>(n)->m_text;
+	return dynamic_cast<const comment *>(n) != nullptr and
+	       m_text == static_cast<const comment *>(n)->m_text;
 }
 
-node* comment::clone() const
+node *comment::clone() const
 {
 	return new comment(m_text);
 }
 
-node* comment::move()
+node *comment::move()
 {
 	return new comment(std::move(*this));
 }
 
-void comment::write(std::ostream& os, format_info fmt) const
+void comment::write(std::ostream &os, format_info fmt) const
 {
 	// if (fmt.indent_width != 0)
-	// 	os << std::endl << std::string(fmt.indent_width, ' ');
+	// 	os << '\n' << std::string(fmt.indent_width, ' ');
 
 	if (not fmt.suppress_comments)
 	{
 		os << "<!--";
-		
+
 		bool lastWasHyphen = false;
-		for (char ch: m_text)
+		for (char ch : m_text)
 		{
 			if (ch == '-' and lastWasHyphen)
 				os << ' ';
-			
+
 			os << ch;
 			lastWasHyphen = ch == '-';
 
@@ -277,52 +316,52 @@ void comment::write(std::ostream& os, format_info fmt) const
 			// 		os << ' ';
 			// }
 		}
-		
+
 		os << "-->";
-		
+
 		if (fmt.indent_width != 0)
-			os << std::endl;
+			os << '\n';
 	}
 }
 
 // --------------------------------------------------------------------
 // processing_instruction
 
-bool processing_instruction::equals(const node* n) const
+bool processing_instruction::equals(const node *n) const
 {
-	return
-		dynamic_cast<const processing_instruction*>(n) != nullptr and
-		m_text == static_cast<const processing_instruction*>(n)->m_text;
+	return dynamic_cast<const processing_instruction *>(n) != nullptr and
+	       m_text == static_cast<const processing_instruction *>(n)->m_text;
 }
 
-node* processing_instruction::clone() const
+node *processing_instruction::clone() const
 {
 	return new processing_instruction(m_target, m_target);
 }
 
-node* processing_instruction::move()
+node *processing_instruction::move()
 {
 	return new processing_instruction(std::move(*this));
 }
 
-void processing_instruction::write(std::ostream& os, format_info fmt) const
+void processing_instruction::write(std::ostream &os, format_info fmt) const
 {
 	if (fmt.indent)
-		os << std::endl << std::string(fmt.indent_level * fmt.indent_width, ' ');
+		os << '\n'
+		   << std::string(fmt.indent_level * fmt.indent_width, ' ');
 
 	os << "<?" << m_target << ' ' << m_text << "?>";
-	
+
 	if (fmt.indent != 0)
-		os << std::endl;
+		os << '\n';
 }
 
 // --------------------------------------------------------------------
 // text
 
-bool text::equals(const node* n) const
+bool text::equals(const node *n) const
 {
 	bool result = false;
-	auto t = dynamic_cast<const text*>(n);
+	auto t = dynamic_cast<const text *>(n);
 
 	if (t != nullptr)
 	{
@@ -341,9 +380,9 @@ bool text::equals(const node* n) const
 bool text::is_space() const
 {
 	bool result = true;
-	for (auto ch: m_text)
+	for (auto ch : m_text)
 	{
-		if (not (ch == ' ' or ch == '\t' or ch == '\n' or ch == '\r'))
+		if (not(ch == ' ' or ch == '\t' or ch == '\n' or ch == '\r'))
 		{
 			result = false;
 			break;
@@ -352,17 +391,17 @@ bool text::is_space() const
 	return result;
 }
 
-node* text::clone() const
+node *text::clone() const
 {
 	return new text(m_text);
 }
 
-node* text::move()
+node *text::move()
 {
 	return new text(std::move(*this));
 }
 
-void text::write(std::ostream& os, format_info fmt) const
+void text::write(std::ostream &os, format_info fmt) const
 {
 	write_string(os, m_text, fmt.escape_white_space, fmt.escape_double_quote, false, fmt.version);
 }
@@ -370,46 +409,46 @@ void text::write(std::ostream& os, format_info fmt) const
 // --------------------------------------------------------------------
 // cdata
 
-bool cdata::equals(const node* n) const
+bool cdata::equals(const node *n) const
 {
-	return
-		dynamic_cast<const cdata*>(n) != nullptr and
-		m_text == static_cast<const cdata*>(n)->m_text;
+	return dynamic_cast<const cdata *>(n) != nullptr and
+	       m_text == static_cast<const cdata *>(n)->m_text;
 }
 
-node* cdata::clone() const
+node *cdata::clone() const
 {
 	return new cdata(m_text);
 }
 
-node* cdata::move()
+node *cdata::move()
 {
 	return new cdata(std::move(*this));
 }
 
-void cdata::write(std::ostream& os, format_info fmt) const
+void cdata::write(std::ostream &os, format_info fmt) const
 {
 	if (fmt.indent)
-		os << std::endl << std::string(fmt.indent_level * fmt.indent_width, ' ');
+		os << '\n'
+		   << std::string(fmt.indent_level * fmt.indent_width, ' ');
 
 	os << "<![CDATA[" << m_text << "]]>";
-	
+
 	if (fmt.indent)
-		os << std::endl;
+		os << '\n';
 }
 
 // --------------------------------------------------------------------
 // attribute
 
-bool attribute::equals(const node* n) const
+bool attribute::equals(const node *n) const
 {
 	bool result = false;
-	const attribute* a = dynamic_cast<const attribute*>(n);
+	const attribute *a = dynamic_cast<const attribute *>(n);
 
 	if (a != nullptr)
 	{
 		result = m_qname == a->m_qname and
-				 m_value == a->m_value;
+		         m_value == a->m_value;
 	}
 	return result;
 }
@@ -422,27 +461,28 @@ std::string attribute::uri() const
 	return m_value;
 }
 
-node* attribute::clone() const
+node *attribute::clone() const
 {
 	return new attribute(m_qname, m_value, m_id);
 }
 
-node* attribute::move()
+node *attribute::move()
 {
 	return new attribute(std::move(*this));
 }
 
-void attribute::write(std::ostream& os, format_info fmt) const
+void attribute::write(std::ostream &os, format_info fmt) const
 {
 	if (fmt.indent_width != 0)
-		os << std::endl << std::string(fmt.indent_width, ' ');
+		os << '\n'
+		   << std::string(fmt.indent_width, ' ');
 	else
 		os << ' ';
 
 	os << m_qname << "=\"";
-	
+
 	write_string(os, m_value, fmt.escape_white_space, true, false, fmt.version);
-	
+
 	os << '"';
 }
 
@@ -473,7 +513,7 @@ void attribute::write(std::ostream& os, format_info fmt) const
 // void name_space::write(std::ostream& os, format_info fmt) const
 // {
 // 	if (fmt.indent_width != 0)
-// 		os << std::endl << std::string(fmt.indent_width, ' ');
+// 		os << '\n' << std::string(fmt.indent_width, ' ');
 // 	else
 // 		os << ' ';
 
@@ -482,9 +522,9 @@ void attribute::write(std::ostream& os, format_info fmt) const
 // 	else
 // 		os << "xmlns:" << m_prefix;
 // 	os << "=\"";
-	
+
 // 	write_string(os, m_uri, fmt.escape_white_space, false, fmt.version);
-	
+
 // 	os << '"';
 // }
 
@@ -497,24 +537,24 @@ element::element()
 {
 }
 
-element::element(const std::string& qname)
+element::element(const std::string &qname)
 	: m_qname(qname)
 	, m_nodes(*this)
 	, m_attributes(*this)
 {
 }
 
-element::element(const std::string& qname, std::initializer_list<zeep::xml::attribute> attributes)
+element::element(const std::string &qname, std::initializer_list<zeep::xml::attribute> attributes)
 	: m_qname(qname)
 	, m_nodes(*this)
 	, m_attributes(*this)
 {
-	for (auto& a: attributes)
+	for (auto &a : attributes)
 		set_attribute(a.get_qname(), a.value());
 }
 
 // copy constructor. Copy data and children, but not parent and sibling
-element::element(const element& e)
+element::element(const element &e)
 	: node()
 	, m_qname(e.m_qname)
 	, m_nodes(*this, e.m_nodes)
@@ -522,16 +562,17 @@ element::element(const element& e)
 {
 }
 
-element::element(element&& e)
+element::element(element &&e)
 	: element()
 {
 	swap(e);
 }
 
-element& element::operator=(const element& e)
+element &element::operator=(const element &e)
 {
 	if (this != &e)
 	{
+		m_qname = e.m_qname;
 		m_nodes = e.m_nodes;
 		m_attributes = e.m_attributes;
 	}
@@ -539,7 +580,7 @@ element& element::operator=(const element& e)
 	return *this;
 }
 
-element& element::operator=(element&& e)
+element &element::operator=(element &&e)
 {
 	if (this != &e)
 	{
@@ -554,19 +595,19 @@ element::~element()
 {
 }
 
-void element::swap(element& e) noexcept
+void element::swap(element &e) noexcept
 {
 	std::swap(m_qname, e.m_qname);
 	m_nodes.swap(e.m_nodes);
 	m_attributes.swap(e.m_attributes);
 }
 
-node* element::clone() const
+node *element::clone() const
 {
 	return new element(*this);
 }
 
-node* element::move()
+node *element::move()
 {
 	return new element(std::move(*this));
 }
@@ -587,8 +628,8 @@ std::string element::lang() const
 std::string element::id() const
 {
 	std::string result;
-	
-	for (auto& a: m_attributes)
+
+	for (auto &a : m_attributes)
 	{
 		if (a.is_id())
 		{
@@ -596,11 +637,11 @@ std::string element::id() const
 			break;
 		}
 	}
-	
+
 	return result;
 }
 
-std::string element::get_attribute(const std::string& qname) const
+std::string element::get_attribute(const std::string &qname) const
 {
 	std::string result;
 
@@ -611,23 +652,24 @@ std::string element::get_attribute(const std::string& qname) const
 	return result;
 }
 
-void element::set_attribute(const std::string& qname, const std::string& value)
+void element::set_attribute(const std::string &qname, const std::string &value)
 {
 	m_attributes.emplace(qname, value);
 }
 
-bool element::equals(const node* n) const
+bool element::equals(const node *n) const
 {
 	bool result = false;
-	const element* e = dynamic_cast<const element*>(n);
+	const element *e = dynamic_cast<const element *>(n);
 
 	if (e != nullptr)
 	{
 		result = name() == e->name() and get_ns() == e->get_ns();
 
-		const node* a = m_nodes.m_head;;
-		const node* b = e->m_nodes.m_head;
-		
+		const node *a = m_nodes.m_head;
+		;
+		const node *b = e->m_nodes.m_head;
+
 		while (a != nullptr or b != nullptr)
 		{
 			if (a != nullptr and b != nullptr and a->equals(b))
@@ -637,16 +679,16 @@ bool element::equals(const node* n) const
 				continue;
 			}
 
-			const text* t;
-			
-			t = dynamic_cast<const text*>(a);
+			const text *t;
+
+			t = dynamic_cast<const text *>(a);
 			if (t != nullptr and t->is_space())
 			{
 				a = a->m_next;
 				continue;
 			}
 
-			t = dynamic_cast<const text*>(b);
+			t = dynamic_cast<const text *>(b);
 			if (t != nullptr and t->is_space())
 			{
 				b = b->m_next;
@@ -674,7 +716,7 @@ bool element::equals(const node* n) const
 				{
 					if (ai == as.end() and bi == bs.end())
 						break;
-					
+
 					if (ai != as.end() and ai->is_namespace())
 					{
 						nsa.insert(ai->value());
@@ -714,9 +756,9 @@ std::string element::get_content() const
 {
 	std::string result;
 
-	for (auto& n: m_nodes)
+	for (auto &n : m_nodes)
 	{
-		auto t = dynamic_cast<const text*>(&n);
+		auto t = dynamic_cast<const text *>(&n);
 		if (t != nullptr)
 			result += t->get_text();
 	}
@@ -724,12 +766,12 @@ std::string element::get_content() const
 	return result;
 }
 
-void element::set_content(const std::string& s)
+void element::set_content(const std::string &s)
 {
 	// remove all existing text nodes (including cdata ones)
 	for (auto n = m_nodes.begin(); n != m_nodes.end(); ++n)
 	{
-		if (dynamic_cast<text*>(&*n) != nullptr)
+		if (dynamic_cast<text *>(&*n) != nullptr)
 			n = m_nodes.erase(n);
 	}
 
@@ -740,24 +782,24 @@ void element::set_content(const std::string& s)
 std::string element::str() const
 {
 	std::string result;
-	
-	for (auto& n: m_nodes)
+
+	for (auto &n : m_nodes)
 		result += n.str();
-	
+
 	return result;
 }
 
-void element::add_text(const std::string& s)
+void element::add_text(const std::string &s)
 {
-	text* textNode = dynamic_cast<text*>(m_nodes.m_tail);
-	
-	if (textNode != nullptr and dynamic_cast<cdata*>(textNode) == nullptr)
+	text *textNode = dynamic_cast<text *>(m_nodes.m_tail);
+
+	if (textNode != nullptr and dynamic_cast<cdata *>(textNode) == nullptr)
 		textNode->append(s);
 	else
 		m_nodes.emplace_back(text(s));
 }
 
-void element::set_text(const std::string& s)
+void element::set_text(const std::string &s)
 {
 	set_content(s);
 }
@@ -767,17 +809,17 @@ void element::flatten_text()
 	auto n = m_nodes.m_head;
 	while (n != m_nodes.m_tail)
 	{
-		auto tn = dynamic_cast<text*>(n);
+		auto tn = dynamic_cast<text *>(n);
 		if (tn == nullptr)
 		{
 			n = n->m_next;
 			continue;
 		}
 
-		if (n->m_next == nullptr)	// should never happen
+		if (n->m_next == nullptr) // should never happen
 			break;
-		
-		auto ntn = dynamic_cast<text*>(n->m_next);
+
+		auto ntn = dynamic_cast<text *>(n->m_next);
 		if (ntn == nullptr)
 		{
 			n = n->m_next;
@@ -789,7 +831,7 @@ void element::flatten_text()
 	}
 }
 
-void element::write(std::ostream& os, format_info fmt) const
+void element::write(std::ostream &os, format_info fmt) const
 {
 	// if width is set, we wrap and indent the file
 	size_t indentation = fmt.indent_level * fmt.indent_width;
@@ -797,7 +839,7 @@ void element::write(std::ostream& os, format_info fmt) const
 	if (fmt.indent)
 	{
 		if (fmt.indent_level > 0)
-			os << std::endl;
+			os << '\n';
 		os << std::string(indentation, ' ');
 	}
 
@@ -807,13 +849,13 @@ void element::write(std::ostream& os, format_info fmt) const
 	auto attr_fmt = fmt;
 	attr_fmt.indent_width = 0;
 
-	for (auto& attr: attributes())
+	for (auto &attr : attributes())
 	{
 		attr.write(os, attr_fmt);
 		if (attr_fmt.indent_width == 0 and fmt.indent_attributes)
 			attr_fmt.indent_width = indentation + 1 + m_qname.length() + 1;
 	}
-	
+
 	if ((fmt.html and kEmptyHTMLElements.count(m_qname)) or
 		(not fmt.html and fmt.collapse_tags and nodes().empty()))
 		os << "/>";
@@ -824,20 +866,21 @@ void element::write(std::ostream& os, format_info fmt) const
 		++sub_fmt.indent_level;
 
 		bool wrote_element = false;
-		for (auto& n: nodes())
+		for (auto &n : nodes())
 		{
 			n.write(os, sub_fmt);
-			wrote_element = dynamic_cast<const element*>(&n) != nullptr;
+			wrote_element = dynamic_cast<const element *>(&n) != nullptr;
 		}
 
 		if (wrote_element and fmt.indent != 0)
-			os << std::endl << std::string(indentation, ' ');
+			os << '\n'
+			   << std::string(indentation, ' ');
 
 		os << "</" << m_qname << '>';
 	}
 }
 
-std::ostream& operator<<(std::ostream& os, const element& e)
+std::ostream &operator<<(std::ostream &os, const element &e)
 {
 	auto flags = os.flags({});
 	auto width = os.width(0);
@@ -852,15 +895,15 @@ std::ostream& operator<<(std::ostream& os, const element& e)
 	return os;
 }
 
-std::string element::namespace_for_prefix(const std::string& prefix) const
+std::string element::namespace_for_prefix(const std::string &prefix) const
 {
 	std::string result;
-	
-	for (auto& a: m_attributes)
+
+	for (auto &a : m_attributes)
 	{
 		if (not a.is_namespace())
 			continue;
-		
+
 		if (a.name() == "xmlns")
 		{
 			if (prefix.empty())
@@ -877,23 +920,23 @@ std::string element::namespace_for_prefix(const std::string& prefix) const
 			break;
 		}
 	}
-	
-	if (result.empty() and dynamic_cast<element*>(m_parent) != nullptr)
-		result = static_cast<element*>(m_parent)->namespace_for_prefix(prefix);
-	
+
+	if (result.empty() and dynamic_cast<element *>(m_parent) != nullptr)
+		result = static_cast<element *>(m_parent)->namespace_for_prefix(prefix);
+
 	return result;
 }
 
-std::pair<std::string,bool> element::prefix_for_namespace(const std::string& uri) const
+std::pair<std::string, bool> element::prefix_for_namespace(const std::string &uri) const
 {
 	std::string result;
 	bool found = false;
 
-	for (auto& a: m_attributes)
+	for (auto &a : m_attributes)
 	{
 		if (not a.is_namespace())
 			continue;
-		
+
 		if (a.value() == uri)
 		{
 			found = true;
@@ -902,14 +945,14 @@ std::pair<std::string,bool> element::prefix_for_namespace(const std::string& uri
 			break;
 		}
 	}
-	
-	if (not found and dynamic_cast<element*>(m_parent) != nullptr)
-		std::tie(result, found) = static_cast<element*>(m_parent)->prefix_for_namespace(uri);
-	
+
+	if (not found and dynamic_cast<element *>(m_parent) != nullptr)
+		std::tie(result, found) = static_cast<element *>(m_parent)->prefix_for_namespace(uri);
+
 	return make_pair(result, found);
 }
 
-void element::move_to_name_space(const std::string& prefix, const std::string& uri,
+void element::move_to_name_space(const std::string &prefix, const std::string &uri,
 	bool recursive, bool including_attributes)
 {
 	// first some sanity checks
@@ -922,11 +965,11 @@ void element::move_to_name_space(const std::string& prefix, const std::string& u
 	else
 	{
 		bool set = false;
-		for (auto& a: m_attributes)
+		for (auto &a : m_attributes)
 		{
 			if (not a.is_namespace())
 				continue;
-			
+
 			if (a.get_qname().length() > 6 and a.get_qname().substr(6) == prefix)
 			{
 				set = true;
@@ -937,25 +980,25 @@ void element::move_to_name_space(const std::string& prefix, const std::string& u
 
 		if (not set)
 			m_attributes.emplace(prefix.empty() ? "xmlns" : "xmlns:" + prefix, uri);
-	}	
+	}
 
 	set_qname(prefix, name());
 
 	if (including_attributes)
 	{
 		// first process the namespace attributes...
-		for (auto& attr: m_attributes)
+		for (auto &attr : m_attributes)
 		{
 			if (not attr.is_namespace())
 				continue;
-			
+
 			auto nsp = prefix_for_namespace(attr.uri());
 			if (not nsp.second)
 				attr.set_qname("xmlns", nsp.first);
 		}
 
 		// ... and then the others, makes sure the namespaces are known
-		for (auto& attr: m_attributes)
+		for (auto &attr : m_attributes)
 		{
 			if (attr.is_namespace())
 				continue;
@@ -976,7 +1019,7 @@ void element::move_to_name_space(const std::string& prefix, const std::string& u
 
 	if (recursive)
 	{
-		for (element& e: *this)
+		for (element &e : *this)
 			e.move_to_name_space(prefix, uri, true, including_attributes);
 	}
 }
@@ -986,24 +1029,21 @@ element_set element::find(const char *path) const
 	return xpath(path).evaluate<element>(*this);
 }
 
-element* element::find_first(const char *path) const
+element::iterator element::find_first(const char *path)
 {
 	element_set s = xpath(path).evaluate<element>(*this);
 
-	element *result = nullptr;
-	if (not s.empty())
-		result = s.front();
-	return result;
+	return s.empty() ? end() : iterator(*this, s.front());
 }
 
 void element::validate()
 {
 	node::validate();
 
-	for (auto& n: nodes())
+	for (auto &n : nodes())
 		n.validate();
 
-	for (auto& a: attributes())
+	for (auto &a : attributes())
 	{
 		if (a.parent() != this)
 			throw exception("validation error: attribute has incorrect parent");
@@ -1012,13 +1052,13 @@ void element::validate()
 
 // --------------------------------------------------------------------
 
-void fix_namespaces(element& e, element& source, element& dest)
+void fix_namespaces(element &e, element &source, element &dest)
 {
-	std::stack<xml::node*> s;
+	std::stack<xml::node *> s;
 
 	s.push(&e);
 
-	std::map<std::string,std::string> mapped;
+	std::map<std::string, std::string> mapped;
 
 	while (not s.empty())
 	{
@@ -1053,16 +1093,16 @@ void fix_namespaces(element& e, element& source, element& dest)
 			}
 		}
 
-		auto el = dynamic_cast<element*>(n);
+		auto el = dynamic_cast<element *>(n);
 		if (el == nullptr)
 			continue;
 
-		for (auto& c: *el)
+		for (auto &c : *el)
 			s.push(&c);
-		
-		for (auto& a: el->attributes())
+
+		for (auto &a : el->attributes())
 			s.push(&a);
 	}
 }
 
-}
\ No newline at end of file
+} // namespace zeep::xml
